@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/python3
 import json
 import argparse
 import re
@@ -76,9 +76,10 @@ def create_email_list(emails_str):
 
 
 class BWAnalyzer:
-    def __init__(self, file_name, email, admin_emails, rate, percentages):
+    def __init__(self, file_name, email, admin_emails, rate, percentages, print_result=True):
         self.email = create_email_list(email)
         self.admin_emails = create_email_list(admin_emails)
+        self.print_result = print_result
 
         if file_name.endswith(".json"):
             file_name = file_name[:-5]
@@ -88,7 +89,10 @@ class BWAnalyzer:
             noise_file = file_name.replace("bandwidth", "noise")
             with open(noise_file + ".json") as f:
                 self.noise_json = json.load(f)
-                latency_file = file_name.replace("bandwidth", "latency")
+        except:
+            pass
+        try:
+            latency_file = file_name.replace("bandwidth", "latency")
             with open(latency_file + ".json") as f:
                 self.latency_json = json.load(f)
         except:
@@ -107,7 +111,7 @@ class BWAnalyzer:
         self.avg_bandwidth = 0
         self.extreme_bandwidth = 0
         self.PROPER_BANDWIDTH_PERC = 0.97  # default percentage of the maximum bandwidth that is considered a good result
-        self.proper_bandwidth = sys.maxint
+        self.proper_bandwidth = sys.maxsize
         self.proper_nodes = list()
         self.nodes_list = sorted(list(self.bandwidth["Nodes"]), key=lambda e: (len(e), e))
         self.positive_deviation_sum = list()
@@ -124,14 +128,17 @@ class BWAnalyzer:
                                   "jazz": {"node_prefix": "jazz", "bw": 11000}
                                   }
         # max/min deviation from all nodes
-        self.max_deviation = -sys.maxint-1
-        self.min_deviation = sys.maxint
+        self.max_deviation = -sys.maxsize-1
+        self.min_deviation = sys.maxsize
         # email templates
         self.base_bandwidth_template = ""
         self.general_test_summary = ""
         self.bad_nodes_template = ""
         self.extreme_bad_nodes_template = ""
         self.all_nodes_under_baseline_template = ""
+        self.bw = {}
+        self.noise = {}
+        self.latency = {}
 
     # get the right value from the half matrix
     def getCell(self, test_json, i, j):
@@ -160,14 +167,23 @@ class BWAnalyzer:
         avg_val = statistic_dict["avg"]
         min_nodes = statistic_dict["min"][self.NODE]
         max_nodes = statistic_dict["max"][self.NODE]
+        link_word = ''
         if test_name == "bandwidth":
+            link_word = 'between hosts: '
+        if test_name == "efficiency":
+            link_word = 'on host '
             if min_val < avg_val * self.SHOW_PERC:
                 perc_diff =  round((avg_val - min_val) / float(avg_val) * 100)
                 min_of_avg_perc = "\t({}% below the avg)".format(perc_diff)
 
-        msg = "Minimum {0}: {1}{6}\t{2}{7}\nMaximum {0}: {3}{6}\t{4}\n" \
-              "Average {0}: {5}{6}\n".format(test_name, min_val, min_nodes, max_val, max_nodes,
-                                             avg_val, unit, min_of_avg_perc)
+        if isinstance(min_val, int):   
+            msg = "Minimum {0}: {1}{6}\t{8}{2}{7}\nMaximum {0}: {3}{6}\t{8}{4}\n" \
+                  "Average {0}: {5}{6}\n".format(test_name, min_val, min_nodes, max_val, max_nodes,
+                                                 avg_val, unit, min_of_avg_perc, link_word)
+        else:  # for getting two digits after decimal point in case of float (noise/latency) 
+            msg = "Minimum {0}: {1:.3f}{6}\t{8}{2}{7}\nMaximum {0}: {3:.3f}{6}\t{8}{4}\n" \
+                  "Average {0}: {5:.3f}{6}\n".format(test_name, min_val, min_nodes, max_val, max_nodes,
+                                                avg_val, unit, min_of_avg_perc, link_word)
         return msg
 
     def noiseStatistics(self):
@@ -219,6 +235,7 @@ class BWAnalyzer:
             max_dict[self.RESULT] = int(max_dict[self.RESULT])
             min_dict[self.RESULT] = int(min_dict[self.RESULT])
             avg = int(avg)
+                      
         ret_val = {"max": max_dict, "min": min_dict, "avg": avg}
         return ret_val
 
@@ -226,6 +243,10 @@ class BWAnalyzer:
         self.statistics_dict["bandwidth"] = self.pairwiseStatistics(self.bandwidth)
         try:
             self.statistics_dict["latency"] = self.pairwiseStatistics(self.latency_json)
+        except:
+            pass
+
+        try:
             self.statistics_dict["noise"] = self.noiseStatistics()
         except:
             pass
@@ -243,11 +264,11 @@ class BWAnalyzer:
         # bad_bandwidth nodes - nodes with bad bandwidth with all nodes
         # proper_nodes - nodes with proper width with all nodes
         # calculation of the deviation sum and min/max between all nodes
-        max_dev = -sys.maxint-1
-        min_dev = sys.maxint
+        max_dev = -sys.maxsize-1
+        min_dev = sys.maxsize
         for i in range(0, self.nodes_num):
             proper_nodes_count = 0
-            lowestbw = sys.maxint
+            lowestbw = sys.maxsize
             count_of_bad_bandwidth = 0
             self.positive_deviation_sum.append({'dev_sum': 0, 'lowestbw' : 0})
             for j in range(0, self.nodes_num):
@@ -279,7 +300,7 @@ class BWAnalyzer:
         # and did not count in the bad bandwidth node list
         bad_nodes_list = [x['node'] for x in self.final_message_dict['$examined_bad_nodes']]
         for i in range(0, self.nodes_num):
-            lowestbw = sys.maxint
+            lowestbw = sys.maxsize
             count_of_extreme_bad_bandwidth = 0
             if self.nodes_list[i] not in bad_nodes_list:
                 for j in range(0, self.nodes_num):
@@ -317,7 +338,7 @@ class BWAnalyzer:
         return template
 
     def get_known_baseline_by_cluster_name(self, cluster_name):
-        result = sys.maxint
+        result = sys.maxsize
         try:
             result = self.KNOWN_CLUSTERS_BW[cluster_name]["bw"]
         except:
@@ -325,7 +346,7 @@ class BWAnalyzer:
         return result
 
     def get_known_baseline_by_node_prefix(self):
-        result = sys.maxint
+        result = sys.maxsize
         nodes_str = hostlist.collect_hostlist(list(self.bandwidth['Nodes'].keys()))
         for cluster_name, cluster_dict in self.KNOWN_CLUSTERS_BW.items():
             try:
@@ -379,7 +400,7 @@ class BWAnalyzer:
 
     def create_templates(self):
 
-        self.bad_nodes_template = """Nodes exhibited poor performance with all other nodes:
+        self.bad_nodes_template = """Nodes which exhibited poor performance with all other nodes:
 minimal bandwidth of $min_bandwidth MB/s
 $bad_nodes_list"""
 
@@ -408,6 +429,8 @@ $statistics"""
 Known baseline bandwidth of cluster is ~$baseline MB/s
 
 """
+    def get_analysis(self):
+        return self.bw, self.noise, self.latency
 
     def prepare_email(self):
         # prepare the email message according to the statistics and data collected
@@ -462,22 +485,30 @@ Known baseline bandwidth of cluster is ~$baseline MB/s
                     extreme_bad_node_msg += self.resolve_tokens(extreme_bad_node_dict, self.extreme_bad_nodes_template)
 
         bandwidth_statistics = self.statistic_dict_to_msg(self.statistics_dict["bandwidth"], "bandwidth", " MB/sec")
+         
         noise_statistics = ""
         latency_statistics = ""
         try:
-            noise_statistics = self.statistic_dict_to_msg(self.statistics_dict["noise"], "efficiency", "")
-            latency_statistics = self.statistic_dict_to_msg(self.statistics_dict["latency"], "latency", " usec")
+            if "noise" in self.statistics_dict:
+                noise_statistics = self.statistic_dict_to_msg(self.statistics_dict["noise"], "efficiency", "")
+            if "latency" in self.statistics_dict:
+                latency_statistics = self.statistic_dict_to_msg(self.statistics_dict["latency"], "latency", " usec")
         except:
             pass
 
         bandwidth_dict = {'$examined_nodes': msg_dict['$examined_nodes'], '$bandwidth_statistics': bandwidth_statistics,
                           '$bad_node_msg': bad_node_msg, '$extreme_bad_node_msg': extreme_bad_node_msg}
+        self.bw  = bandwidth_dict 
+        
         result_msg = self.resolve_tokens(bandwidth_dict, self.base_bandwidth_template)
-        if latency_statistics != "":
+        if noise_statistics != "":
             noise_dict = {'$test_name': 'Noise', '$statistics': noise_statistics}
-            latency_dict = {'$test_name': 'Latency', '$statistics': latency_statistics}
             result_msg += self.resolve_tokens(noise_dict, self.general_test_summary)
+            self.noise = noise_dict
+        if latency_statistics != "":
+            latency_dict = {'$test_name': 'Latency', '$statistics': latency_statistics}
             result_msg += self.resolve_tokens(latency_dict, self.general_test_summary)
+            self.latency = latency_dict
 
         output_dir = os.path.dirname(self.file_name)
         if output_dir == "":
@@ -529,10 +560,14 @@ Known baseline bandwidth of cluster is ~$baseline MB/s
         self.check_known_nodes_under_baseline()
 
         subject, file_list, result_msg = self.prepare_email()
-        print(result_msg)
+        if self.print_result: #  case for direct running (not via generate_output.py)
+            print(result_msg)
+
         if self.email:
             send_email(subject, result_msg, self.email, file_list)
 
+        return result_msg    
+
 
 if __name__ == "__main__":
     parser = argparse.ArgumentParser()
