@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2023 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2005 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -430,7 +431,7 @@ static inline const cl_fmap_item_t *cl_fmap_end(IN const cl_fmap_t *
 *	cl_fmap_end is useful for determining the validity of map items returned
 *	by cl_fmap_head, cl_fmap_tail, cl_fmap_next, or cl_fmap_prev.  If the
 *	map item pointer returned by any of these functions compares to the end,
-*	the end of the map was encoutered.
+*	the end of the map was encountered.
 *	When using cl_fmap_head or cl_fmap_tail, this condition indicates that
 *	the map is empty.
 *
@@ -568,7 +569,7 @@ static inline cl_fmap_item_t *cl_fmap_prev(IN const cl_fmap_item_t *
 * RETURN VALUES
 *	Pointer to the map item with the next lower key value in a flexi map.
 *
-*	Pointer to the map end if the specifid item was the first item in
+*	Pointer to the map end if the specified item was the first item in
 *	the flexi map.
 *
 * SEE ALSO
@@ -595,12 +596,12 @@ cl_fmap_item_t *cl_fmap_insert(IN cl_fmap_t * const p_map,
 *
 *	p_key
 *		[in] Pointer to the key value to assign to the item.  Storage
-*		for the key must be persistant, as only the pointer is stored.
+*		for the key must be persistent, as only the pointer is stored.
 *		Users are responsible for maintaining the validity of key
 *		pointers while they are in use.
 *
 *	p_item
-*		[in] Pointer to a cl_fmap_item_t stucture to insert into the flexi map.
+*		[in] Pointer to a cl_fmap_item_t structure to insert into the flexi map.
 *
 * RETURN VALUE
 *	Pointer to the item in the map with the specified key.  If insertion
