@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2021 Nvidia, Inc. All rights reserved.
  * Copyright (c) 2004, 2005 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2005 Mellanox Technologies LTD. All rights reserved.
@@ -171,6 +172,9 @@ typedef struct _cl_u64_vector {
 *	state
 *		State of the U64 vector.
 *
+*	sorted
+*		The U64 vector is sorted.  (For lookup)
+*
 * SEE ALSO
 *	U64 Vector
 *********/
@@ -219,7 +223,7 @@ cl_u64_vector_init(IN cl_u64_vector_t * const p_vector,
 /*
 * PARAMETERS
 *	p_vector
-*		[in] Pointer to a cl_u64_vector_t structure to inititalize.
+*		[in] Pointer to a cl_u64_vector_t structure to initialize.
 *
 *	min_size
 *		[in] Initial number of elements.
@@ -802,6 +806,50 @@ void cl_u64_vector_sort(IN cl_u64_vector_t * const p_vector);
 *	cl_u64_vector_sort, cl_u64_vector_lookup.
 *********/
 
+/****f* Component Library: U64 Vector/cl_u64_vector_sort_cmp
+* NAME
+*	cl_u64_vector_sort
+*
+* DESCRIPTION
+*	The cl_u64_vector_sort_cmp sorts elements in a U64 vector by using the
+*	user supplied function as the comparator.
+*
+* SYNOPSIS
+*/
+cl_status_t cl_u64_vector_sort_cmp(IN cl_u64_vector_t * const p_vector,
+				   IN size_t offset,
+				   IN size_t length,
+				   IN int (*cmp)(const void *, const void *));
+/*
+* PARAMETERS
+*	p_vector
+*		[in] Pointer to a cl_u64_vector_t.
+*	offset
+*		[in] Offset in vector to start the sort.
+*	length
+*		[in] Length of vector slice to sort.  If length is zero, sort
+*		     the remainder of the vector.
+*	cmp
+*		[in] qsort compatible comparison function.
+*
+* RETURN VALUE
+*	CL_SUCCESS
+*		Sort was performed successfully.
+*
+*	CL_INVALID_PARAMETER
+*		offset is greater than or equal to the vector size.
+*		length is greater than the vector size.
+*		(offset+length) is greater than the vector size.
+*
+* NOTES
+*	cl_u64_vector_sort_user sorts U64 vector in time complexity of O(N*log(N)),
+*	where N is length parameter.
+*
+* SEE ALSO
+*	U64 Vector, cl_u64_vector_find_from_start, cl_u64_vector_apply_func,
+*	cl_u64_vector_sort, cl_u64_vector_lookup.
+*********/
+
 /****f* Component Library: U64 Vector/cl_u64_vector_lookup
 * NAME
 *	cl_u64_vector_lookup
@@ -839,5 +887,37 @@ cl_u64_vector_lookup(IN const cl_u64_vector_t * const p_vector,
 *	cl_u64_vector_sort.
 *********/
 
+/****f* Component Library: U64 Vector/cl_u64_vector_qcontains
+* NAME
+*	cl_u64_vector_qcontains
+*
+* DESCRIPTION
+*	The cl_u64_vector_qcontains function returns whether a sorted U64 vector
+*	contains an element.
+*
+* SYNOPSIS
+*/
+boolean_t cl_u64_vector_qcontains(IN const cl_u64_vector_t * const p_vector,
+				  IN const uint64_t element);
+/*
+* PARAMETERS
+*	p_vector
+*		[in] Pointer to a cl_u64_vector_t structure to query.
+*
+*	element
+*		[in] Element to search.
+*
+* RETURN VALUES
+*	TRUE if the element is found, FALSE otherwise.
+*
+* NOTES
+*	1. cl_u64_vector_qcontains does not remove the found element from
+*	the U64 vector.
+*	2. The vector must be sorted when calling this function.
+*
+* SEE ALSO
+*	U64 Vector, cl_u64_vector_find_from_start, cl_u64_vector_find_from_end.
+*********/
+
 END_C_DECLS
 #endif				/* _CL_U64_VECTOR_H_ */
