@@ -1,6 +1,6 @@
 /*
  * Copyright (c) 2004-2021 Mellanox Technologies LTD. All rights reserved.
- * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  *
  * This software is available to you under the terms of the
  * OpenIB.org BSD license included below:
@@ -161,6 +161,27 @@ struct GreaterFwVerObjComparer{
     }
 };
 
+//check that PortInfo.CapabilityMask's 'IsExtendedSpeedsSupported' is set
+#define IS_SUPPORT_PORT_INFO_EXT_SPEEDS(cap_mask) ((cap_mask) & 0x4000)
+
+//check that PortInfo.CapabilityMask's 'IsCapabilityMask2Supported' is set
+#define IS_SUPPORT_PORT_INFO_CAP_MASK2(cap_mask) ((cap_mask & 0x8000))
+
+//check that PortInfo.CapabilityMask2's IsPortInfoExtendedSupported is set
+#define IS_SUPPORT_PORT_INFO_EXTENDED(cap_mask) ((cap_mask) & 0x2)
+
+//check that PortInfo.CapabilityMask2's IsSwitchPortStateTableSupported is set
+#define IS_SUPPORT_SWITCH_PORT_STATE_TABLE(cap_mask) ((cap_mask) & 0x0008)
+
+//check that PortInfo.CapabilityMask's 'IsHierarchyInfoSupported' is set
+#define IS_SUPPORT_HIERARCHY_INFO(cap_mask) ((cap_mask & 0x80000000))
+
+//check that PortInfo.CapabilityMsk's IsLinkRoundTripLatencySupported is set
+#define IS_SUPPORT_LINK_ROUND_TRIP_LATENCY(cap_mask) ((cap_mask) & 0x1000000)
+
+//check that PortInfo.CapabilityMask2's 'IsExtendedSpeeds2Supported' is set
+#define IS_SUPPORT_PORT_INFO_EXT_SPEEDS2(cap_mask2) ((cap_mask2) & 0x800)
+
 enum VLArbitrationBlockIndex {
     LOWER_LOW_PRIORITY_VL_ARBITRATION_TABLE    = 1,
     UPPER_LOW_PRIORITY_VL_ARBITRATION_TABLE    = 2,
@@ -168,11 +189,35 @@ enum VLArbitrationBlockIndex {
     UPPER_HIGH_PRIORITY_VL_ARBITRATION_TABLE   = 4
 };
 
+enum FastRecoveryCountersTrigger {
+    FR_TRIGGER_CREDIT_WD_TIMEOUT = 2,
+    FR_TRIGGER_FIRST = FR_TRIGGER_CREDIT_WD_TIMEOUT,
+    FR_TRIGGER_RESERVED = 3,
+    FR_TRIGGER_RAW_BER = 4,
+    FR_TRIGGER_EFFECTIVE_BER = 5,
+    FR_TRIGGER_SYMBOL_BER = 6,
+    FR_TRIGGER_LAST
+};
+
+enum ProfilesConfigFeature {
+    FAST_RECOVERY_PROFILE = 0,
+    QOS_VL_CONFIG_PROFILES,
+    DATA_TYPE_TO_VL_PROFILES,
+    NVLHBF_PROFILES
+};
+
+#define ENTRY_PLANE_FILTER_EGRESS_BLOCK_SIZE 256
+#define RAIL_FILTER_EGRESS_BLOCK_SIZE        256
+#define CROCODILE_MAX_PLANES    4
+#define BM_MAX_PLANES           1
+
+
 // reasons for bad direct route
 typedef enum {
     IBDIAG_BAD_DR_NONE = 0,
     IBDIAG_BAD_DR_NODE_FIRST,
     IBDIAG_BAD_DR_NODE_NODE_INFO = IBDIAG_BAD_DR_NODE_FIRST,
+    IBDIAG_BAD_DR_NODE_GUID_INVALID,
     IBDIAG_BAD_DR_NODE_DUPLICATE_CHECK,
     IBDIAG_BAD_DR_NODE_NODE_DESCRIPTION,
     IBDIAG_BAD_DR_NODE_INCONSISTENT, //data db inconsistency, no mem
@@ -200,7 +245,8 @@ typedef struct IbdiagBadDirectRoute {
         reason = IBDIAG_BAD_DR_NONE;
         port_num = 0;
         message = "";
-    }
+    };
+    IbdiagBadDirectRoute() { clear(); }
 } IbdiagBadDirectRoute_t;
 
 #define NUM_CAPABILITY_FIELDS 4
@@ -255,6 +301,26 @@ typedef vector < struct adaptive_routing_info * >               vector_p_vs_ar_i
 typedef vector < struct port_rn_counters* >                     vector_p_vs_rn_counters;
 typedef vector < struct port_routing_decision_counters* >       vector_p_vs_routing_decision_counters;
 typedef vector < struct hbf_config * >                          vector_p_hbf_config;
+typedef vector < struct VS_CreditWatchdogTimeoutCounters * >    vector_p_vs_credit_wd_timeout_counters;
+typedef vector < struct VS_FastRecoveryCounters * >             vector_p_vs_fast_recovery_counters;
+typedef vector < vector_p_vs_fast_recovery_counters >           vec_vec_p_vs_fast_recovery_counters;
+typedef vector < struct VS_PortGeneralCounters * >              vec_p_vs_port_general_counters;
+typedef vector < struct SMP_ProfilesConfig * >                  vector_p_smp_profiles_config;
+typedef vector < vector_p_smp_profiles_config >                 vec_vec_p_smp_profiles_config;
+typedef vector < struct SMP_CreditWatchdogConfig * >            vector_p_credit_watchdog_config;
+typedef vector < vector_p_credit_watchdog_config >              vec_vec_p_credit_watchdog_config;
+typedef vector < struct SMP_BERConfig * >                       vector_p_ber_config;
+typedef vector < vector_p_ber_config >                          vec_vec_p_ber_config;
+typedef vector < struct VS_PerformanceHistogramBufferControl *> vec_p_performance_histogram_buffer_control;
+typedef vector < vec_p_performance_histogram_buffer_control >   vec_vec_performance_histogram_buffer_control;
+typedef vector < struct VS_PerformanceHistogramInfo * >         vec_p_performance_histogram_info;
+typedef vector < struct VS_PerformanceHistogramBufferData * >   vec_p_performance_histogram_buffer_data;
+typedef vector < vec_p_performance_histogram_buffer_data >      vec_vec_performance_histogram_buffer_data;
+typedef vector < struct VS_PerformanceHistogramPortsControl * > vec_p_performance_histogram_ports_control;
+typedef vector < vec_p_performance_histogram_ports_control >    vec_v_performance_histogram_ports_control;
+
+typedef vector < struct VS_PerformanceHistogramPortsData * >    vec_p_vs_performance_histogram_ports_data;
+typedef vector < vec_p_vs_performance_histogram_ports_data >    vec_v_vs_performance_histogram_ports_data;
 
 // pFRN
 typedef vector < struct SMP_pFRNConfig * >                      vector_p_pfrn_config;
@@ -264,8 +330,10 @@ typedef vector < struct Class_C_KeyInfo * >                     vector_p_n2n_key
 
 typedef vector < struct SMP_PortInfo * >                        vector_p_smp_port_info;
 typedef vector < struct SMP_PortInfoExtended * >                vector_p_smp_port_info_ext;
+typedef vector < struct ib_extended_node_info* >                vector_p_smp_mlnx_ext_node_info;
 typedef vector < struct SMP_MlnxExtPortInfo * >                 vector_p_smp_mlnx_ext_port_info;
 typedef vector < struct VendorSpec_GeneralInfo * >              vector_p_vs_general_info;
+typedef vector < struct VS_SwitchNetworkInfo * >                vector_p_vs_switch_network_info;
 typedef vector < struct SMP_SwitchPortStateTable >              vector_smp_switch_port_state_table;
 typedef vector < u_int16_t * >                                  vector_p_pm_cap_mask;
 typedef vector < IB_ClassPortInfo * >                           vector_p_class_port_info;
@@ -278,21 +346,31 @@ typedef vector < struct SMP_VPortInfo * >                       vector_p_smp_vpo
 typedef vector < struct SMP_VNodeInfo * >                       vector_p_smp_vnode_info;
 typedef vector < struct SMP_TempSensing * >                     vector_p_smp_temp_sensing;
 typedef vector < struct SMP_QosConfigSL * >                     vector_p_smp_qos_config_sl;
+typedef vector < struct SMP_QosConfigVL * >                     vector_p_smp_qos_config_vl;
 typedef vector < struct SMP_RouterInfo * >                      vector_p_smp_router_info;
 typedef vector < struct SMP_AdjSiteLocalSubnTbl * >             vector_p_smp_adj_router_tbl;
 typedef vector < vector_p_smp_adj_router_tbl >                  vector_v_smp_adj_router_tbl;
 typedef vector < struct SMP_NextHopTbl * >                      vector_p_smp_next_hop_router_tbl;
 typedef vector < vector_p_smp_next_hop_router_tbl >             vector_v_smp_next_hop_router_tbl;
-typedef vector < u_int64_t >                                    vector_guids;
+typedef vector < struct SMP_AdjSubnetsRouterLIDInfoTable * >    vector_p_smp_adj_subnet_router_lid_info_tbl;
+typedef vector < vector_p_smp_adj_subnet_router_lid_info_tbl >  vector_v_smp_adj_subnet_router_lid_info_tbl;
+typedef vector < struct SMP_RouterLIDTable * >                  vector_p_smp_router_lid_tbl;
+typedef vector < vector_p_smp_router_lid_tbl >                  vector_v_smp_router_lid_tbl;
+typedef vector < struct SMP_ARGroupToRouterLIDTable * >         vector_p_smp_ar_group_router_lid_tbl;
+typedef vector < vector_p_smp_ar_group_router_lid_tbl >         vector_v_ar_group_router_lid_tbl;
+
+
+typedef vector < u_int64_t >                                    vector_uint64;
 typedef list < IBNode * >                                       list_p_node;
-typedef list < IBPort * >                                       list_p_port;
 typedef list < sm_info_obj_t * >                                list_p_sm_info_obj;
 typedef list < direct_route_t  * >                              list_p_direct_route;
+typedef vector < direct_route_t  * >                            vector_p_direct_route;
 typedef list < IbdiagBadDirectRoute * >                         list_p_bad_direct_route;
 typedef list < string >                                         list_string;
 typedef list < u_int8_t >                                       list_uint8;
 typedef list < index_line_t >                                   list_index_line;
 typedef map < u_int64_t, list_p_direct_route >                  map_guid_list_p_direct_route;
+typedef map < pair < u_int64_t, u_int8_t >, direct_route_t * >  map_port_p_direct_route;
 typedef map < u_int16_t, list_p_port >                          map_lid_list_p_port;
 typedef map < string,  u_int64_t >                              map_str_uint64;
 typedef map < u_int8_t,  u_int64_t >                            map_uint8_uint64;
@@ -338,6 +416,42 @@ typedef vector < struct CC_CongestionHCARPParameters * >        vec_cc_hca_rp_pa
 typedef vector < struct CC_CongestionHCANPParameters * >        vec_cc_hca_np_parameters;
 typedef vector < struct CC_CongestionHCAStatisticsQuery * >     vec_cc_hca_statistics_query;
 
+// CC algo MADs
+typedef vector < struct CC_CongestionHCAAlgoConfig * >          vec_cc_hca_algo_config;
+typedef vector < vec_cc_hca_algo_config>                        vec_vec_cc_hca_algo_config;
+typedef vector < struct CC_CongestionHCAAlgoConfigParams * >    vec_cc_hca_algo_config_params;
+typedef vector < vec_cc_hca_algo_config_params >                vec_vec_cc_hca_algo_config_params;
+typedef vector < struct CC_CongestionHCAAlgoCounters * >        vec_cc_hca_algo_counters;
+typedef vector < vec_cc_hca_algo_counters>                      vec_vec_cc_hca_algo_counters;
+
+// NVL MADs
+typedef vector < struct SMP_AnycastLIDInfo * >                  vec_p_nvl_anycast_lid_info;
+typedef vector < vec_p_nvl_anycast_lid_info >                   vec_v_nvl_anycast_lid_info;
+typedef vector < struct SMP_NVLHBFConfig * >                    vec_p_nvl_hbf_config;
+typedef vector < struct SMP_ContainAndDrainInfo * >             vec_p_nvl_contain_and_drain_info;
+typedef vector < vec_p_nvl_contain_and_drain_info >             vec_v_nvl_contain_and_drain_info;
+typedef vector < struct SMP_ContainAndDrainPortState * >        vec_p_nvl_contain_and_drain_port_state;
+typedef vector < vec_p_nvl_contain_and_drain_port_state >       vec_v_nvl_contain_and_drain_port_state;
+
+
+// NVL - Reduction
+typedef vector < struct IB_ClassPortInfo*>                      vec_p_nvl_class_port_info;
+typedef vector < struct NVLReductionInfo*>                      vec_p_nvl_reduction_info;
+typedef vector < struct NVLReductionPortInfo*>                  vec_p_nvl_reduction_port_info;
+typedef vector < struct NVLReductionRoundingMode*>              vec_p_nvl_reduction_rounding_mode;
+typedef vector < struct NVLReductionForwardingTable * >            vec_p_nvl_reduction_forwarding_table;
+typedef vector < vec_p_nvl_reduction_forwarding_table >         vec_v_nvl_reduction_forwarding_table;
+typedef vector < struct NVLPenaltyBoxConfig * >                    vec_p_nvl_penalty_box_config;
+typedef vector < vec_p_nvl_penalty_box_config >                 vec_v_nvl_penalty_box_config;
+typedef vector < struct NVLReductionConfigureMLIDMonitors * >   vec_p_nvl_reduction_configure_mlid_monitors;
+typedef vector < struct NVLReductionCounters * >                vec_p_nvl_reduction_counters;
+typedef vector < vec_p_nvl_reduction_counters >                 vec_v_nvl_reduction_counters;
+
+typedef vector < struct SMP_LinearForwardingTableSplit * >      vec_p_smp_lft_split;
+
+typedef vector < struct SMP_ExtendedSwitchInfo * >              vec_p_smp_extended_switch_info;
+typedef vector < struct PM_PortSamplesResult * >                vec_p_pm_port_samples_result;
+typedef vector < struct SMP_ChassisInfo* >                      vec_p_smp_chassis_info;
 
 //SMP Capability Mask bits
 //note: when adding new values, update the 'first' and the 'last'
@@ -356,7 +470,7 @@ typedef enum {
     EnSMPCapIsAccessRegisterSupported,                 //10
     EnSMPCapIsInterProcessCommunicationSupported,      //11
     EnSMPCapIsPortSLToPrivateLFTMapSupported,          //12
-    EnSMPCapIsExtendedSwitchInfoSupported,             //13
+    EnSMPCapIsExtendedNodeInfoSupported,               //13
     EnSMPCapIsASlaveVirtualSwitch,                     //14
     EnSMPCapIsVirtualizationSupported,                 //15
     EnSMPCapIsAdvancedRetransAndFECModesSupported,     //16
@@ -374,52 +488,105 @@ typedef enum {
     EnSMPCapIsEnhancedTrap128Supported,                //28
     EnSMPCapIsDevInfoSupported,                        //29
     EnSMPCapIsGlobalOOOSupported,                      //30
-    // todo - add missing bits
-    EnSMPCapIsVSCCSupported = 44,                      //44
-    EnSMPCapLast = EnSMPCapIsVSCCSupported
+    EnSMPCapIsMADAddlStatusSupported,                  //31
+    EnSMPCapIsQoSConfigSLRateLimitPortSupported,       //32
+    EnSMPCapIsQoSConfigSLRateLimitVPortSupported,      //33
+    EnSMPCapIsIsQoSConfigSLAllocBWPortSupported,       //34
+    EnSMPCapIsQoSConfigSLAllocBWVPortSupported,        //35
+    EnSMPCapIsQoSConfigHighPriorityVLGroupRateLimitSupported,  //36
+    EnSMPCapIsQoSConfigHighPriorityVLGroupAllocBWSupported,    //37
+    EnSMPCapsIsQoSConfigLowPriorityVLGroupRateLimitSupported,  //38
+    EnSMPCapIsQoSConfigLowPriorityVLGroupAllocBWSupported,     //39
+    EnSMPCapIsQoSConfigVLRateLimitSupported,           //40
+    EnSMPCapIsQoSConfigVLAllocBWSupported,             //41
+    //the bit is RESERVED                              //42
+    EnSMPCapIsCableInfoBankNumberSupported = 43,       //43
+    EnSMPCapIsVSCCSupported,                           //44
+    EnSMPCapIsAdaptiveTimeoutSupported,                //45
+    EnSMPCapIsClassEnableSupported,                    //46
+    EnSMPCapIsExtendedAddressModeSupported,            //47
+    EnSMPCapIsRouterLIDSupported,                      //48
+    EnSMPCapIsLinearForwardingTableSplitSupported,     //49
+    EnSMPCapIsDataTypetoSLMappingSupported,            //50
+    EnSMPCapIsQoSConfigVLSupported,                    //51
+    EnSMPCapIsFastRecoverySupported,                   //52
+    EnSMPCapIsCreditWatchdogSupported,                 //53
+    EnSMPCapIsNVLFabricManagementSupported,            //54
+    EnSMPCapIsNVLReductionManagementSupported,         //55
+    EnSMPCapIsRailFilterSupported,                     //56
+    EnSMPCapIsNVLHBFConfigSupported,                   //57
+    EnSMPCapIsDataTypetoVLMappingSupported,            //58 should be reserved ?
+    EnSMPCapIsContainAndDrainSupported,                //59
+    EnSMPCapIsProfilesConfigSupported,                 //60
+    EnSMPCapIsBERConfigSupported,                      //61
+    EnSMPCapIsGlobalOOOV2Supported,                    //62
+    EnSMPCapIsVLBufferWeightSupported,                 //63
+    EnSMPCapIsEndPortPlaneFilterConfigSupported,       //64
+    EnSMPCapIsEntryPlaneFilterSupported,               //65
+    EnSMPCapIsBwUtiliztionSupported,                   //66
+    EnSMPCapIsExtendedSwitchIfoSupported,              //67
+    EnSMPCapIsFabricManagerInfoSupported,              //68
+    EnSMPCapIsExtPortInfoUnhealthyReasonSupported,     //69
+    EnSMPCapIsMulticastPrivateLinearForwardingSupported,  //70
+    EnSMPCapIsPortSLToMulticastPrivateLFTMapSupported, //71
+    EnSMPCapIsChassisInfoSupported,                    //72
+    EnSMPCapIsPLRTriggerSupported,                     //73
+    EnSMPCapIsFwISSUSupported,                         //74
+    EnSMPCapLast = EnSMPCapIsFwISSUSupported
 } EnSMPCapabilityMaskBit_t;
 
 //GMP Capability Mask bits
 //note: when adding new values, update the 'first' and the 'last'
 typedef enum {
-    EnGMPCapIsPortPowerStateSupported = 0,
+    EnGMPCapIsPortPowerStateSupported = 0,             //   vs-spec::IsPortPowerStateSupported
     EnGMPCapFirst = EnGMPCapIsPortPowerStateSupported,
-    EnGMPCapIsDeviceSoftResetSupported,                //1
-    EnGMPCapIsExtPortAccessSupported,                  //2
-    EnGMPCapIsPhyConfigSupported,                      //3
-    EnGMPCapIsVendSpecLedControlSupported,             //4
-    EnGMPCapIsConfigSpaceAccessSupported,              //5
-    EnGMPCapIsPerVLCountersSupported,                  //6
-    EnGMPCapIsPerVLExtendedSupported,                  //7
-    EnGMPCapIsPortLLRStatisticsSupported,              //8
-    EnGMPCapIsCounterGroupsSupported,                  //9
-    EnGMPCapIsEnhancedConfigSpaceAccessSupported,      //10
-    EnGMPCapIsFlowMonitoringSupported,                 //11
-    EnGMPCapIsCongestionNotificationSupported,         //12
-    EnGMPCapIsVirtualSwitchSupported,                  //13
-    EnGMPCapIsPortXmitQLengthSupported,                //14
-    EnGMPCapIsMulticastForwardingTableSupported,       //15
-    EnGMPCapIsPortXmitWaitVLExtendedSupported,         //16
-    EnGMPCapIsExtI2CPortAccessSupported,               //17
-    EnGMPCapIsDiagnosticDataSupported,                 //18
-    EnGMPCapIsMaxRetransmissionRateSupported,          //19
-    EnGMPCapIsAccessRegisterSupported,                 //20
-    EnGMPCapIsPortHistogramsSupported,                 //21
-    EnGMPCAPIsPoolHistogramsSupported,                 //22
-    EnGMPCAPIsVPortCountersSupported,                  //23
-    EnGMPCAPIsDiagnosticDataIdentificationSupported,   //24
-    EnGMPCAPIsEnhPort0DeviceResetSupported,            //25
-    EnGMPCAPIsDevInfoSupported,                        //26
-    EnGMPCAPIsMADAddlStatusSupported,                  //27
-    EnGMPCAPIsCableBurnSupported,                      //28
-    EnGMPCAPIsMirroringEventsSupported,                //29
-    EnGMPCAPIsBERMonitoringSupported,                  //30
-    EnGMPCAPIsBERMonitoringEventsSupported,            //31
-    EnGMPIsDCQCNSupported,                             //32
-    EnGMPCAPIsPLRMaxRetransmissionRateSupported,       //33
-    EnGMPCAPIsEffectiveCounterSupported,               //34
-    EnGMPCAPIsRawBerPerLaneSupported,                  //35
-    EnGMPCapLast = EnGMPCAPIsRawBerPerLaneSupported
+    EnGMPCapIsDeviceSoftResetSupported,                //1  vs-spec::IsDeviceSoftResetSupported
+    EnGMPCapIsExtPortAccessSupported,                  //2  vs-spec::IsExtGPIOPortAccessSupported
+    EnGMPCapIsPhyConfigSupported,                      //3  vs-spec::IsPhyConfigSupported
+    EnGMPCapIsVendSpecLedControlSupported,             //4  vs-spec::IsVendSpecLedControlSupported
+    EnGMPCapIsConfigSpaceAccessSupported,              //5  vs-spec::IsConfigSpaceAccessSupported
+    EnGMPCapIsPerVLCountersSupported,                  //6  vs-spec::IsPerVLCountersSupported
+    EnGMPCapIsPerVLExtendedSupported,                  //7  vs-spec::IsPerVLExtendedSupported
+    EnGMPCapIsPortLLRStatisticsSupported,              //8  vs-spec::IsPortLLRStatisticsSupported
+    EnGMPCapIsCounterGroupsSupported,                  //9  vs-spec::IsCounterGroupsSupported
+    EnGMPCapIsEnhancedConfigSpaceAccessSupported,      //10 vs-spec::IsEnhancedConfigSpaceAccess-Supported
+    EnGMPCapIsFlowMonitoringSupported,                 //11 vs-spec::IsFlowMonitoringSupported
+    EnGMPCapIsCongestionNotificationSupported,         //12 vs-spec::IsCongestionNotificationSupported
+    EnGMPCapIsVirtualSwitchSupported,                  //13 vs-spec::IsVirtualSwitchSupported
+    EnGMPCapIsPortXmitQLengthSupported,                //14 vs-spec::IsPortXmitQLengthSupported
+    EnGMPCapIsMulticastForwardingTableSupported,       //15 vs-spec::IsMulticastForwardingTableSupported
+    EnGMPCapIsPortXmitWaitVLExtendedSupported,         //16 vs-spec::IsPortXmitWaitVLExtendedSupported
+    EnGMPCapIsExtI2CPortAccessSupported,               //17 vs-spec::IsExtI2CPortAccessSupported
+    EnGMPCapIsDiagnosticDataSupported,                 //18 vs-spec::IsDiagnosticDataSupported
+    EnGMPCapIsMaxRetransmissionRateSupported,          //19 vs-spec::IsMaxRetransmissionRateSupported
+    EnGMPCapIsAccessRegisterSupported,                 //20 vs-spec::IsAccessRegisterGMPSupported
+    EnGMPCapIsPortHistogramsSupported,                 //21 vs-spec::IsPortHistogramsSupported
+    EnGMPCAPIsPoolHistogramsSupported,                 //22 vs-spec::IsPoolHistogramsSupported
+    EnGMPCAPIsVPortCountersSupported,                  //23 vs-spec::IsVPortCountersSupported
+    EnGMPCAPIsDiagnosticDataIdentificationSupported,   //24 vs-spec::IsDiagnosticDataIdentification-Supported
+    EnGMPCAPIsEnhPort0DeviceResetSupported,            //25 vs-spec::IsEnhPort0DeviceResetSupported
+    EnGMPCAPIsDevInfoSupported,                        //26 vs-spec::IsDevInfoSupported
+    EnGMPCAPIsMADAddlStatusSupported,                  //27 vs-spec::IsMADAddlStatusSupported
+    EnGMPCAPIsCableBurnSupported,                      //28 vs-spec::IsCableBurnSupported
+    EnGMPCAPIsMirroringEventsSupported,                //29 vs-spec::IsMirroringEventsSupported
+    EnGMPCAPIsBERMonitoringSupported,                  //30 vs-spec::IsBERMonitoringSupported
+    EnGMPCAPIsBERMonitoringEventsSupported,            //31 vs-specIsBERMonitoringEventsSupported
+    EnGMPIsDCQCNSupported,                             //32 vs-spec::IsDCQCNSupported
+    EnGMPCAPIsPLRMaxRetransmissionRateSupported,       //33 vs-spec::IsPLRMaxRetransmissionRate-Supported
+    EnGMPCAPIsEffectiveCounterSupported,               //34 vs-spec::IsEffectiveCounterSupported
+    EnGMPCAPIsRawBerPerLaneSupported,                  //35 vs-spec::IsRawBerPerLaneSupported
+    EnGMPCAPIsGMPAccessRegisterSupported,              //36 vs-spec::IsAccessRegisterSupported
+    EnGMPCAPIsMirroringInfoSupported,                  //37 vs-spec::IsMirroringInfoSupported
+    EnGMPCAPIsExtendedAddressModeSupported,            //38 vs-spec::IsExtendedAddressModeSupported
+    EnGMPCAPIsSwitchNetworkInfoSupported,              //39 vs-spec::IsSwitchNetworkInfoSupported
+    EnGMPCAPIsCreditWatchdogTimeoutCountersSupported,  //40 vs-spec::IsCreditWatchdogTimeoutCountersSupported
+    EnGMPCAPIsFastRecoveryCountersSupported,           //41 vs-spec::IsFastRecoveryCountersSupported
+    EnGMPCAPIsGeneralResetSupported,                   //42 vs-spec::IsGeneralResetSupported
+    EnGMPCAPIsBufferHistogramSupported,                //43 vs-spec::IsBufferHistogramSupported
+    EnGMPCAPIsPortsHistogramSupported,                 //44 vs-spec::IsPortsHistogramSupported
+    EnGMPCAPIsPacketBufferAllocControlSupported,       //45 vs-spec::IsPacketBufferAllocControlSupported
+    EnGMPCAPIsPortGeneralCountersSupported,            //46 vs-spec::IsPortGeneralCountersSupported
+    EnGMPCapLast = EnGMPCAPIsPortGeneralCountersSupported
 } EnGMPCapabilityMaskBit_t;
 
 typedef enum {
@@ -433,7 +600,8 @@ typedef enum {
 } EnSpecialPortsCapabilityMaskBit_t;
 
 enum EnCCCapabilityMaskBit {
-    EnCCHCAStatisticsQuerySupported = 1
+    EnCCHCAStatisticsQuerySupported = 1,
+    EnCCHCA_AlgoConfigParamCounetrsSupported = 2
 };
 
 enum PMClassPortInfoCapabilityMask {
@@ -453,6 +621,13 @@ enum PMClassPortInfoCapabilityMask2 {
     IsVirtualizationSupported
 };
 
+enum NVLClassPortInfoCapabilityMask {
+    NVL_CPI_CapMas_IsVendKeySupported = 8,
+    NVL_CPI_CapMas_IsMultipleManagersSupported,
+    NVL_CPI_CapMas_IsPenaltyBoxSupported
+};
+
+
 
 class CapabilityModule;
 class Ibis;
@@ -490,7 +665,7 @@ public:
         m_mask_first_bit(mask_first_bit), m_mask_last_bit(mask_last_bit) {};
     virtual ~CapabilityMaskConfig() {};
 
-    virtual int Init(Ibis* ibis_obj);
+    virtual int Init();
     //Init mask for SwitchX, ConnectX3, Golan.
     //These devices have old fw that doesn't support capability mask,
     //so store the mask for them, that tells what capabilities are supported
@@ -521,7 +696,7 @@ public:
 
     bool IsMaskKnown(u_int64_t guid);
 
-    bool IsSupportedCapability(IBNode *node, u_int8_t cap_bit);
+    bool IsSupportedCapability(const IBNode *node, u_int8_t cap_bit) const;
     bool IsLongestPrefixMatch(u_int64_t guid, u_int8_t &prefix_len,
                               u_int64_t &matched_guid, query_or_mask &qmask);
     int DumpCapabilityMaskFile(ostream& sout);
@@ -541,7 +716,7 @@ public:
     virtual void InitFWConnectX3(fw_version_obj_t &fw);
     //Init FW version, from which connectIB supports SMP capability mask
     virtual void InitFWConnectIB(fw_version_obj_t &fw);
-    virtual int Init(Ibis* ibis_obj);
+    virtual int Init();
     void DumpCSVVSGeneralInfo(stringstream &sout);
 };
 
@@ -555,7 +730,7 @@ public:
     virtual void InitFWConnectX3(fw_version_obj_t &fw);
     //Init FW version, from which connectIB supports GMP capability mask
     virtual void InitFWConnectIB(fw_version_obj_t &fw);
-    virtual int Init(Ibis* ibis_obj);
+    virtual int Init();
 };
 
 class CapabilityModule {
@@ -564,7 +739,7 @@ class CapabilityModule {
 public:
     CapabilityModule() {}
     ~CapabilityModule() {}
-    int Init(Ibis *ibis_obj);
+    int Init();
 
     inline SmpMask &GetSmpMask() { return smp_mask; };
     inline GmpMask &GetGmpMask() { return gmp_mask; };
@@ -581,7 +756,7 @@ public:
     bool IsSMPUnsupportedMadDevice(uint32_t ven_id, device_id_t dev_id,
                                    capability_mask_t &mask);
     bool IsSMPMaskKnown(u_int64_t guid);
-    bool IsSupportedSMPCapability(IBNode *node, u_int8_t cap_bit);
+    bool IsSupportedSMPCapability(const IBNode *node, u_int8_t cap_bit) const;
 
     bool IsLongestSMPPrefixMatch(u_int64_t guid, u_int8_t &prefix_len,
                                  u_int64_t &matched_guid,
@@ -602,7 +777,7 @@ public:
     int GetGMPFwConfiguredMask(uint32_t ven_id, device_id_t dev_id,
                                fw_version_obj_t &fw, capability_mask_t &mask,
                                bool *is_only_fw = NULL);
-    bool IsSupportedGMPCapability(IBNode *node, u_int8_t cap_bit);
+    bool IsSupportedGMPCapability(const IBNode *node, u_int8_t cap_bit) const;
 
     int GetFw(u_int64_t guid, fw_version_obj_t &fw);
 
@@ -636,7 +811,6 @@ public:
 };
 
 
-
 /****************************************************/
 /* sections defines */
 #define SECTION_NODES                "NODES"
@@ -645,11 +819,15 @@ public:
 #define SECTION_PORT_HIERARCHY_INFO  "PORT_HIERARCHY_INFO"
 #define SECTION_PHYSICAL_HIERARCHY_INFO  "PHYSICAL_HIERARCHY_INFO"
 #define SECTION_FEC_MODE             "FEC_MODE"
+#define SECTION_EXTENDED_NODE_INFO   "EXTENDED_NODE_INFO"
 #define SECTION_EXTENDED_PORT_INFO   "EXTENDED_PORT_INFO"
+#define SECTION_EXTENDED_SWITCH_INFO "EXTENDED_SWITCH_INFO"
+#define SECTION_CHASSIS_INFO         "CHASSIS_INFO"
 #define SECTION_PORT_INFO_EXTENDED   "PORT_INFO_EXTENDED"
 #define SECTION_LINKS                "LINKS"
 #define SECTION_PM_INFO              "PM_INFO"
 #define SECTION_PM_PORT_SAMPLES_CONTROL  "PM_PORT_SAMPLES_CONTROL"
+#define SECTION_PM_PORT_SAMPLES_RESULT   "PM_PORT_SAMPLES_RESULT"
 #define SECTION_PM_DELTA             "PM_DELTA"
 #define SECTION_SM_INFO              "SM_INFO"
 #define SECTION_NODES_INFO           "NODES_INFO"
@@ -660,23 +838,31 @@ public:
 #define SECTION_VPORTS               "VPORTS"
 #define SECTION_VPORTS_GUID_INFO     "VPORTS_GUID_INFO"
 #define SECTION_QOS_CONFIG_SL        "QOS_CONFIG_SL"
+#define SECTION_QOS_CONFIG_VL        "QOS_CONFIG_VL"
 #define SECTION_VPORTS_QOS_CONFIG_SL "VPORTS_QOS_CONFIG_SL"
 #define SECTION_TEMP_SENSING         "TEMP_SENSING"
 #define SECTION_BER_TEST             "BER_TEST"
 #define SECTION_PHY_TEST             "PHY_TEST"
 #define SECTION_ROUTERS_INFO         "ROUTERS_INFO"
 #define SECTION_ROUTERS_ADJ_TBL      "ROUTERS_ADJ_SITE_LOCAL_SUBNETS_TABLE"
+#define SECTION_ROUTERS_ADJ_LID_TBL  "ROUTERS_ADJ_SUBNETS_LID_INFO_TABLE"
 #define SECTION_ROUTERS_NEXT_HOP_TBL "ROUTERS_NEXT_HOP_TABLE"
 #define SECTION_SHARP_AN_INFO        "SHARP_AN_INFO"
 #define SECTION_SHARP_PM_COUNTERS    "SHARP_PM_COUNTERS"
 #define SECTION_HBA_PORT_COUNTERS    "HBA_PORT_COUNTERS"
 #define SECTION_GENERAL_INFO_SMP     "GENERAL_INFO_SMP"
 #define SECTION_AR_INFO              "AR_INFO"
+#define SECTION_PORT_DRS             "PORT_DRS"
 #define SECTION_HBF_CONFIG           "HBF_CONFIG"
 #define SECTION_VL_ARBITRATION_TBL   "VL_ARBITRATION_TABLE"
 #define SECTION_WHBF_CONFIG          "WHBF_CONFIG"
 #define SECTION_RN_COUNTERS          "RN_COUNTERS"
 #define SECTION_HBF_PORT_COUNTRERS   "HBF_PORT_COUNTERS"
+#define SECTION_STAGE_PERF_INFO      "STAGE_PERF_INFO"
+#define SECTION_CSV_PERF_INFO        "CSV_PERF_INFO"
+#define SECTION_SWITCH_NETWORK_INFO  "SWITCH_NETWORK_INFO"
+#define SECTION_IBIS_MAD_INFO        "IBIS_MAD_INFO"
+#define SECTION_IBIS_PERF_INFO       "IBIS_PERF_INFO"
 
 // pFRN headers
 #define SECTION_PFRN_CONFIG          "PFRN_CONFIG"
@@ -693,6 +879,41 @@ public:
 #define SECTION_CC_HCA_RP_PARAMETERS          "CC_HCA_RP_PARAMETERS"
 #define SECTION_CC_HCA_NP_PARAMETERS          "CC_HCA_NP_PARAMETERS"
 #define SECTION_CC_HCA_STATISTICS_QUERY       "CC_HCA_STATISTICS_QUERY"
+// CC algo
+#define SECTION_CC_HCA_ALGO_CONFIG_SUP        "CC_HCA_ALGO_CONFIG_SUPPORT"
+#define SECTION_CC_HCA_ALGO_CONFIG            "CC_HCA_ALGO_CONFIG"
+#define SECTION_CC_HCA_ALGO_CONFIG_PARAMS     "CC_HCA_ALGO_CONFIG_PARAMS"
+#define SECTION_CC_HCA_ALGO_COUNTERS          "CC_HCA_ALGO_COUNTERS"
+
+// fast recovery
+#define SECTION_CREDIT_WATCHDOG_TIMEOUT_COUNTERS  "CREDIT_WATCHDOG_TIMEOUT_COUNTERS"
+#define SECTION_FAST_RECOVERY_COUNTERS            "FAST_RECOVERY_COUNTERS"
+#define SECTION_PROFILES_CONFIG                   "PROFILES_CONFIG"
+#define SECTION_CREDIT_WATCHDOG_CONFIG            "CREDIT_WATCHDOG_CONFIG"
+#define SECTION_BER_CONFIG                        "BER_CONFIG"
+#define SECTION_PORT_GENERAL_COUNTERS             "PORT_GENERAL_COUNTERS"
+
+//Histogram Info
+#define SECTION_PERFORMANCE_HISTOGRAM_INFO           "PERFORMANCE_HISTOGRAM_INFO"
+#define SECTION_PERFORMANCE_HISTOGRAM_BUFFER_CONTROL "PERFORMANCE_HISTOGRAM_BUFFER_CONTROL"
+#define SECTION_PERFORMANCE_HISTOGRAM_BUFFER_DATA    "PERFORMANCE_HISTOGRAM_BUFFER_DATA"
+#define SECTION_PERFORMANCE_HISTOGRAM_PORTS_CONTROL  "PERFORMANCE_HISTOGRAM_PORTS_CONTROL"
+#define SECTION_PERFORMANCE_HISTOGRAM_PORTS_DATA     "PERFORMANCE_HISTOGRAM_PORTS_DATA"
+
+// NVLink
+#define SECTION_NVL_ANYCAST_LID_INFO                    "NVL_ANYCAST_LID_INFO"
+#define SECTION_NVL_HBF_CONFIG                          "NVL_HBF_CONFIG"
+#define SECTION_NVL_CONTAIN_AND_DRAIN_INFO              "NVL_CONTAIN_AND_DRAIN_INFO"
+#define SECTION_NVL_CONTAIN_AND_DRAIN_PORT_STATE        "NVL_CONTAIN_AND_DRAIN_PORT_STATE"
+#define SECTION_NVL_CLASS_PORT_INFO                     "NVL_CLASS_PORT_INFO"
+#define SECTION_NVL_REDUCTION_INFO                      "NVL_REDUCTION_INFO"
+#define SECTION_NVL_REDUCTION_FORWARDING_TABLE          "NVL_REDUCTION_FORWARDING_TABLE"
+#define SECTION_NVL_PENALTY_BOX_CONFIG                  "NVL_PENALTY_BOX_CONFIG"
+#define SECTION_NVL_REDUCTION_PORT_INFO                 "NVL_REDUCTION_PORT_INFO"
+#define SECTION_NVL_REDUCTION_CONFIGURE_MLID_MONITORS   "NVL_REDUCTION_CONFIGURE_MLID_MONITORS"
+#define SECTION_NVL_REDUCTION_COUNTERS                  "NVL_REDUCTION_COUNTERS"
+#define SECTION_NVL_REDUCTION_ROUNDING_MODE             "NVL_REDUCTION_ROUNDING_MODE"
+#define SECTION_LINEAR_FORWARDING_TABLE_SPLIT           "LINEAR_FORWARDING_TABLE_SPLIT"
 
 #define SECTION_COMBINED_CABLE_INFO "CABLE_INFO"
 /****************************************************/
@@ -716,8 +937,7 @@ public:
 #define OPTION_DEF_VAL_TRUE "TRUE"
 #define OPTION_DEF_VAL_FALSE "FALSE"
 
-#define HIERARCHY_TEMPLATE_GUID_PHYSICAL 0x01
-#define HIERARCHY_TEMPLATE_GUID_PORT     0x03
+#define PORT_PROFILES_IN_BLOCK 128
 
 enum OVERFLOW_VALUES {
     OVERFLOW_VAL_NONE   = 0x0,
@@ -747,6 +967,10 @@ enum IBDIAG_RETURN_CODES {
     IBDIAG_ERR_CODE_ILLEGAL_FUNCTION_CALL = 0x15,
     IBDIAG_ERR_CODE_FILE_NOT_OPENED =0x16,
     IBDIAG_ERR_CODE_FILE_NOT_EXIST =0x17,
+    IBDIAG_ERR_CODE_DISABLED = 0x18,
+    IBDIAG_ERR_CODE_PORT_GUID_NOT_FOUND = 0x19,
+    IBDIAG_ERR_CODE_NODE_GUID_NOT_FOUND = 0x20,
+    IBDIAG_ERR_CODE_NOT_SUPPORTED = 0x21,
     IBDIAG_ERR_CODE_LAST
 };
 
@@ -777,22 +1001,20 @@ enum IBDIAG_RETURN_CODES {
     #define INOUT
 #endif
 
-#define ARRAY_SIZE(arr) (sizeof(arr)/sizeof(arr[0]));
+#define ARRAY_SIZE(arr) (sizeof(arr)/sizeof(arr[0]))
 
 /****************************************************/
 /* log Macros */
 #ifdef DEBUG
-    #define IBDIAG_LOG(level, fmt, ...) TT_LOG(TT_LOG_MODULE_IBDIAG, level, fmt, ## __VA_ARGS__);
-    #define IBDIAG_ENTER TT_ENTER( TT_LOG_MODULE_IBDIAG );
-    #define IBDIAG_RETURN(rc) { TT_EXIT( TT_LOG_MODULE_IBDIAG );  \
-                                return (rc); }
-    #define IBDIAG_RETURN_VOID { TT_EXIT( TT_LOG_MODULE_IBDIAG );   \
-                                    return; }
+    #define IBDIAG_LOG(level, fmt, ...) TT_LOG(TT_LOG_MODULE_IBDIAG, level, fmt, ## __VA_ARGS__)
+    #define IBDIAG_ENTER TT_ENTER( TT_LOG_MODULE_IBDIAG )
+    #define IBDIAG_RETURN(rc) { TT_EXIT( TT_LOG_MODULE_IBDIAG ); return (rc); }
+    #define IBDIAG_RETURN_VOID { TT_EXIT( TT_LOG_MODULE_IBDIAG ); return; }
 #else           /*def DEBUG */
     #define IBDIAG_LOG(level, fmt, ...)
     #define IBDIAG_ENTER
-    #define IBDIAG_RETURN(rc) return (rc);
-    #define IBDIAG_RETURN_VOID return;
+    #define IBDIAG_RETURN(rc) return (rc)
+    #define IBDIAG_RETURN_VOID return
 #endif          /*def DEBUG */
 
 #ifndef WIN32
