@@ -1,6 +1,7 @@
 /*
  * Copyright (c) 2020-2020 Mellanox Technologies LTD. All rights reserved.
- *
+ * Copyright (C) 2022-2022, NVIDIA CORPORATION & AFFILIATES. ALL RIGHTS RESERVED.
+ * 
  * This software is available to you under the terms of the
  * OpenIB.org BSD license included below:
  *
@@ -132,6 +133,79 @@ class OutputControl
             friend class OutputControl;
         };
 
+    public:
+        class Aliases
+        {
+            public:
+                typedef std::vector<std::string>         vec_t;
+                typedef std::map<std::string, vec_t>     map_t;
+
+            private:
+                map_t m_data;
+
+            public:
+                inline const map_t & data() const { return m_data; }
+
+            public:
+                Aliases() { }
+
+            private:
+                inline std::string & prepare_key(const std::string & name, std::string & key) const
+                {
+                    std::transform(
+                            name.begin(),
+                            name.end(),
+                            std::back_inserter(key),
+                            ::tolower);
+
+                    return trim(key);
+                }
+
+                inline std::string prepare_key(const std::string & name) const
+                {
+                    std::string key;
+                    return prepare_key(name, key);
+                }
+
+            public:
+                inline vec_t & operator[](const std::string & name) {
+                    return m_data[prepare_key(name)];
+                }
+
+            public:
+                inline bool exist(const std::string & name) const
+                {
+                    map_t::const_iterator x = m_data.find(prepare_key(name));
+                    return x != m_data.end() && !x->second.empty();
+                }
+
+            public:
+                inline size_t size(const std::string & name) const
+                {
+                    map_t::const_iterator x = m_data.find(prepare_key(name));
+                    return x != m_data.end() ? x->second.size() : 0;
+                }
+
+
+            public:
+                std::ostream & output(std::ostream & stream, const std::string & prefix) const
+                {
+                    stream << prefix << "Aliases:" << std::endl;
+
+                    for(map_t::const_iterator x = m_data.begin(); x != m_data.end(); ++x) {
+                        stream << std::left << prefix << '\t' << std::setw(15) << x->first << " : " << std::right;
+                        const char * splitter = "";
+                        for(vec_t::const_iterator y = x->second.begin(); y != x->second.end(); ++y) {
+                            stream << splitter << '\"' << *y << '\"';
+                            splitter = ", ";
+                        }
+                        stream << std::endl;
+                    }
+
+                    return stream;
+                }
+        };
+
         template<typename _Type>
             class Group
             {
@@ -139,100 +213,27 @@ class OutputControl
                     typedef std::map<Identity, _Type> map_t;
 
                 protected:
-                    class Aliases
-                    {
-                        public:
-                            typedef std::vector<std::string>         vec_t;
-                            typedef std::map<std::string, vec_t>     map_t;
-
-                        private:
-                            map_t m_data;
-
-                        public:
-                            inline const map_t & data() const { return m_data; }
-
-                        public:
-                            Aliases() { }
-
-                        private:
-                            inline std::string & prepare_key(const std::string & name, std::string & key) const
-                            {
-                                std::transform(
-                                        name.begin(),
-                                        name.end(),
-                                        std::back_inserter(key),
-                                        ::tolower);
-
-                                return trim(key);
-                            }
-
-                            inline std::string prepare_key(const std::string & name) const
-                            {
-                                std::string key;
-                                return prepare_key(name, key);
-                            }
-
-                        public:
-                            inline vec_t & operator[](const std::string & name) {
-                                return m_data[prepare_key(name)];
-                            }
-
-                        public:
-                            inline bool exist(const std::string & name) const
-                            {
-                                map_t::const_iterator x = m_data.find(prepare_key(name));
-                                return x != m_data.end() && !x->second.empty();
-                            }
-
-                        public:
-                            inline size_t size(const std::string & name) const
-                            {
-                                map_t::const_iterator x = m_data.find(prepare_key(name));
-                                return x != m_data.end() ? x->second.size() : 0;
-                            }
-
-
-                        public:
-                            std::ostream & output(std::ostream & stream, const std::string & prefix)
-                            {
-                                stream << prefix << "Aliases:" << std::endl;
-
-                                for(map_t::const_iterator x = m_data.begin(); x != m_data.end(); ++x) {
-                                    stream << std::left << prefix << '\t' << std::setw(15) << x->first << " : " << std::right;
-                                    const char * splitter = "";
-                                    for(vec_t::const_iterator y = x->second.begin(); y != x->second.end(); ++y) {
-                                        stream << splitter << '\"' << *y << '\"';
-                                        splitter = ", ";
-                                    }
-                                    stream << std::endl;
-                                }
-
-                                return stream;
-                            }
-                    };
+                    const Aliases &  m_aliases;
 
                 protected:
                     std::string      m_name;
                     map_t            m_data;
                     Flags            m_types;
-                    Aliases          m_aliases;
 
                 public:
-                    const std::string & name() const { return m_name; }
-
-                protected:
-                    Aliases & aliases() { return m_aliases; }
+                    const std::string & name() const        { return m_name;    }
+                    const Aliases &     aliases() const     { return m_aliases; }
 
                 public:
-                    Group(const std::string & name, Flags types)
-                        : m_name(name), m_types(types)
+                    Group(const Aliases & aliases, const std::string & name, Flags types)
+                        : m_aliases(aliases), m_name(name), m_types(types)
                     {}
 
                 public:
                     inline bool is_supported(const Identity & identity) const
                     {
                         Flags type = Flags(identity.flags() & OutputControl_Flag_Type_Mask);
-                        return identity.flags() && OutputControl_Flag_Valid &&
+                        return (identity.flags() & OutputControl_Flag_Valid) &&
                                 Flags(m_types & type) == type;
                     }
 
@@ -340,18 +341,20 @@ class OutputControl
                     {
                         if(!identity.is_generic() || identity.is_special())
                             return internal_unset(identity);
-                        else
-                            if(m_aliases.exist(identity.type())) {
-                                bool                        res = true;
-                                typename Aliases::vec_t &   vec = m_aliases[identity.type()];
+                        else {
+                            typename Aliases::map_t::const_iterator x = m_aliases.data().find(identity.type());
 
-                                for(typename Aliases::vec_t::const_iterator x = vec.begin(); x != vec.end(); ++x)
-                                    res &= internal_unset(Identity(*x)).is_valid();
+                            if(x != m_aliases.data().end()) {
+                                bool res = true;
+
+                                for(typename Aliases::vec_t::const_iterator z = x->second.begin(); z != x->second.end(); ++z)
+                                    res &= internal_unset(Identity(*z)).is_valid();
 
                                 return res ? identity : Identity::Null;
                             }
                             else
                                 return internal_unset(identity);
+                        }
                     }
 
                     // UnSet(string) method CAN'T return Identity by reference, because in case multi-alias
@@ -366,18 +369,20 @@ class OutputControl
                     {
                         if(!identity.is_generic() || identity.is_special())
                             return internal_set(identity, value);
-                        else
-                            if(m_aliases.exist(identity.type())) {
-                                bool                        res = true;
-                                typename Aliases::vec_t &   vec = m_aliases[identity.type()];
+                        else {
+                            typename Aliases::map_t::const_iterator x = m_aliases.data().find(identity.type());
 
-                                for(typename Aliases::vec_t::const_iterator x = vec.begin(); x != vec.end(); ++x)
-                                    res &= internal_set(Identity(*x), value).is_valid();
+                            if(x != m_aliases.data().end()) {
+                                bool res = true;
+
+                                for(typename Aliases::vec_t::const_iterator z = x->second.begin(); z != x->second.end(); ++z)
+                                    res &= internal_set(Identity(*z), value).is_valid();
 
                                 return res ? identity : Identity::Null;
                             }
                             else
                                 return internal_set(identity, value);
+                        }
                     }
 
                     // Set(string) method CAN'T return Identity by reference, because in case multi-alias
@@ -478,31 +483,37 @@ class OutputControl
         static AppSettings app_settings;
 
     private:
-        Group<bool>          m_enabled;
-        Group<std::string>   m_pathes;
-        Group<bool>          m_csv_to_file;
-        Group<bool>          m_compression;
-        Group<bool>          m_binary;
-        Group<bool>          m_in_summary;
+        Aliases                     m_aliases;
+
+    private:
+        Group<bool>                 m_enabled;
+        Group<std::string>          m_pathes;
+        Group<bool>                 m_csv_to_file;
+        Group<bool>                 m_compression;
+        Group<bool>                 m_binary;
+        Group<bool>                 m_in_summary;
 
     public:
-        Group<bool> &                enabled()             { return m_enabled;       }
-        const Group<bool> &          enabled() const       { return m_enabled;       }
+        Group<bool> &               enabled()               { return m_enabled;     }
+        const Group<bool> &         enabled() const         { return m_enabled;     }
 
-        Group<std::string> &         pathes()              { return m_pathes;        }
-        const Group<std::string> &   pathes() const        { return m_pathes;        }
+        Group<std::string> &        pathes()                { return m_pathes;      }
+        const Group<std::string> &  pathes() const          { return m_pathes;      }
 
-        Group<bool> &                csv_to_file()         { return m_csv_to_file;   }
-        const Group<bool> &          csv_to_file() const   { return m_csv_to_file;   }
+        Group<bool> &               csv_to_file()           { return m_csv_to_file; }
+        const Group<bool> &         csv_to_file() const     { return m_csv_to_file; }
 
-        Group<bool> &                compression()         { return m_compression;   }
-        const Group<bool> &          compression() const   { return m_compression;   }
+        Group<bool> &               compression()           { return m_compression; }
+        const Group<bool> &         compression() const     { return m_compression; }
 
-        Group<bool> &                binary()              { return m_binary;        }
-        const Group<bool> &          binary() const        { return m_binary;        }
+        Group<bool> &               binary()                { return m_binary;      }
+        const Group<bool> &         binary() const          { return m_binary;      }
 
-        Group<bool> &                in_summary()          { return m_in_summary;    }
-        const Group<bool> &          in_summary() const    { return m_in_summary;    }
+        Group<bool> &               in_summary()            { return m_in_summary;  }
+        const Group<bool> &         in_summary() const      { return m_in_summary;  }
+
+        Aliases &                   aliases()               { return m_aliases;     }
+        const Aliases &             aliases() const         { return m_aliases;     }
 
     private:
         OutputControl();
@@ -522,25 +533,24 @@ class OutputControl
         class Properties
         {
             private:
-                Identity        m_identity;
-                bool            m_is_valid;
-                bool            m_enabled;
-                std::string     filename;
-                std::string     m_path;
-                bool            m_csv_to_file;
-                bool            m_compression;
-                bool            m_binary;
-                bool            m_in_summary;
+                Identity            m_identity;
+                bool                m_is_valid;
+                bool                m_enabled;
+                std::string         m_path;
+                bool                m_csv_to_file;
+                bool                m_compression;
+                bool                m_binary;
+                bool                m_in_summary;
 
             public:
-                Identity        identity() const      { return m_identity;      }
-                bool            is_valid() const      { return m_is_valid;      }
-                bool            enabled() const       { return m_enabled;       }
-                std::string     path() const          { return m_path;          }
-                bool            csv_to_file() const   { return m_csv_to_file;   }
-                bool            compression() const   { return m_compression;   }
-                bool            binary() const        { return m_binary;        }
-                bool            in_summary() const    { return m_in_summary;    }
+                Identity            identity() const      { return m_identity;      }
+                bool                is_valid() const      { return m_is_valid;      }
+                bool                enabled() const       { return m_enabled;       }
+                const std::string & path() const          { return m_path;          }
+                bool                csv_to_file() const   { return m_csv_to_file;   }
+                bool                compression() const   { return m_compression;   }
+                bool                binary() const        { return m_binary;        }
+                bool                in_summary() const    { return m_in_summary;    }
 
             public:
                 Properties(const Identity & identity)
