@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2010-2020 Mellanox Technologies LTD. All rights reserved.
  *
  * This software is available to you under a choice of one of two
@@ -66,16 +66,16 @@ typedef vector <bool> BoolVec;
 typedef map <u_int64_t, u_int16_t> GuidToGroupMap;
 typedef GuidToGroupMap::iterator GuidToGroupMapIter;
 
-struct AnalizeDFSetupData {
+struct AnalyzeDFSetupData {
 	stack <ARSWDataBaseEntry*> m_group_stack;
 	SwDbEntryPrtList m_spines;
 	SwDbEntryPrtList m_leafs;
 	bool             m_group_discovered;
 
-	u_int16_t hca_to_sw_lid_mapping[IB_LID_UCAST_END_HO + 1];
+	u_int16_t hca_to_sw_lid_mapping[IB_MAX_NUM_OF_PLANES_XDR + 1][IB_LID_UCAST_END_HO + 1];
 	DfSwData* sw_lid_to_df_data_mapping[IB_LID_UCAST_END_HO + 1];
 
-	AnalizeDFSetupData() : m_group_discovered(false) {
+	AnalyzeDFSetupData() : m_group_discovered(false) {
 		memset(hca_to_sw_lid_mapping, 0, sizeof(hca_to_sw_lid_mapping));
 		memset(sw_lid_to_df_data_mapping, 0, sizeof(sw_lid_to_df_data_mapping));
 	}
@@ -90,8 +90,6 @@ struct AnalizeDFSetupData {
 	}
 };
 
-static int RankRootSwitches(void *context, uint64_t guid, char *p);
-
 /* ============================================================================ */
 class OSMAdaptiveRoutingManager {
 public:
@@ -209,8 +207,9 @@ public:
 			ib_ar_info_get_grp_tbl_cpy_sup(&sw_db_entry.m_ar_info));
 	}
 
-	bool IsDetourEnabled() {
-		return (m_master_db.m_frn_enable);
+	bool IsGeneratePFRN() {
+		/* Generate PLFTs as requires for PFRN regardless of devices support */
+		return m_master_db.m_pfrn_enable;
 	}
 
 	/**
@@ -222,6 +221,17 @@ public:
 	void SetPortGroups(ARSWDataBaseEntry &sw_db_entry,
 			   GroupsData &groups_data);
 
+	/*
+	 * The allocation of group number to leaf sw is saved
+	 * after the first allocation
+	 */
+	u_int16_t AllocateSwArGroup(uint16_t sw_lid, uint16_t group_cap, uint8_t plft_id);
+
+	/*
+	 * Add switch to list of roots.
+	 * Return 0 on success.
+	 */
+	int AddRootSwitchByGUID(uint64_t guid);
 private:
     /*
      * members
@@ -233,7 +243,7 @@ private:
     OSMThreadPool                   m_thread_pool;
     OSMParallelPortGroupsCalculator m_port_groups_calculator;
 
-    struct ARSWDataBase             m_sw_db;                /** Switchs database */
+    struct ARSWDataBase             m_sw_db;                /** Switches database */
     struct MasterDataBase           m_master_db;            /** Master settings database */
 
     string                          m_conf_file_name;       /** Name of user input file */
@@ -253,6 +263,8 @@ private:
     bool                            m_sw_info_configured;  /** If true should remove sw info configuration
                                                              * if protocol is not DF PLUS*/
 
+    void                            *m_re_context;           /** routing engine specific data */
+
     uint16_t                        m_max_df_group_number;
     GuidToGroupMap                  m_guid_to_dfp_group;
     bool                            m_group_discovered;
@@ -261,6 +273,7 @@ private:
 
     bool                            m_is_init;
     SwDbEntryPrtList                m_root_switches;
+    cl_u64_vector_t		    m_dfp_potential_roots; /** Used by coloring algorithm for DF+ and DF+2 engines */
 
     /*
      * Methods
@@ -416,20 +429,20 @@ private:
     /**
      * Mark all switch as supports / not supports adaptive routing
      *
-     * @return number of unsupportes switches
+     * @return number of unsupported switches
      *
      */
     int ARInfoGetProcess();
 
     /**
-     * Set Required ARInfo R/W values for all supportes switches
+     * Set Required ARInfo R/W values for all supported switches
      *
      */
     void SetRequiredARInfo(ARSWDataBaseEntry &db_entry);
     /**
-     * Get calculated GroupCap from all supportes switches
+     * Get calculated GroupCap from all supported switches
      *
-     * @return number of unsupportes switches
+     * @return number of unsupported switches
      *
      */
     int  ARInfoGetGroupCapProcess();
@@ -437,7 +450,7 @@ private:
     /**
      *  Configure ARInfo on needed ones
      *
-     * @return number of unsupportes switches
+     * @return number of unsupported switches
      *
      */
     int ARInfoSetProcess();
@@ -468,13 +481,13 @@ private:
 	void SetPortGroupsForRn(ARSWDataBaseEntry &sw_db_entry,
 				GroupsData &groups_data);
     /**
-     * Update DB used to copy a single group to multiple groups
+     * Update AR port groups DB per PLFT
      *
      * @param sw_db_entry the entry of the switch in the db
      * @param groups the calculated GroupsPtrVec object
      */
-    void ARUpdateSWGroupToCopy(ARSWDataBaseEntry &sw_db_entry,
-			       GroupsPtrVec &groups);
+    void ARUpdateSWGroupPerPLFT(ARSWDataBaseEntry &sw_db_entry,
+				GroupsPtrVec &groups);
 
     /**
      * Update the switch group table top with the calculated one
@@ -534,11 +547,23 @@ private:
      */
     void ARCalculatePortGroupsParallelLinks();
 
+    /**
+     * Calculate adaptive routing settings for MINHOP
+     */
+    void ARCalculatePortGroupsMinhop();
+
+    /**
+     * Calculate adaptive routing settings for ASYMTREE
+     */
+    int ARCalculatePortGroupsAsymTree();
+
     /**
      * Calculate adaptive routing settings according to fat tree algorithm
      */
     void ARCalculatePortGroupsTree();
 
+    void ARSetUcastPort(ARSWDataBaseEntry &sw_db_entry, PSGroupData &group_data,
+                        uint16_t lid, int first_port_in_group);
     /**
      * Update data structures on adding new lid to AR groups
      */
@@ -584,13 +609,10 @@ private:
      *
      * @param hca_to_sw_lid_mapping
      */
-     int SetHcaToSwLidMapping(osm_physp_t *p_hca_physp,
-                             osm_node_t *p_remote_sw_node,
-                             u_int16_t *hca_to_sw_lid_mapping );
-
-    //The allocation of group number to leaf sw is saved
-    //after the first allocation
-    u_int16_t AllocateSwArGroup(uint16_t sw_lid, uint16_t group_cap);
+     int SetHcaToSwLidMapping(
+		osm_physp_t *p_hca_physp,
+		osm_node_t *p_remote_sw_node,
+		u_int16_t hca_to_sw_lid_mapping[IB_MAX_NUM_OF_PLANES_XDR + 1][IB_LID_UCAST_END_HO + 1]);
 
     /** ----------  DRAGON FLY PLUS ---------- */
 
@@ -606,11 +628,11 @@ private:
     void ClearAllDragonflyConfiguration();
 
     /**
-     * Init DragonFlyPlus Data structures on cycle begining
+     * Init DragonFlyPlus Data structures on cycle beginning
      *
      * @return non zero on failure
      */
-    int InitDragonFlyPlus(AnalizeDFSetupData &setup_data);
+    int InitDragonFlyPlus(AnalyzeDFSetupData &setup_data);
 
     /**
      * Set SUPPORT_DF bit for all switches on fabric that are
@@ -629,20 +651,26 @@ private:
      */
     bool SetRequiredPLFTInfo(ARSWDataBaseEntry *p_sw_entry);
 
-    friend int RankRootSwitches(void *cxt, uint64_t guid, char *p);
-    int AnalyzeDragonFlyPlusTopo(AnalizeDFSetupData &setup_data, const char* root_guid_file);
-    int DragonFlyPlusMarkRanks(AnalizeDFSetupData &setup_data, uint8_t &max_rank);
-    int DragonFlyPlusMarkIslands(AnalizeDFSetupData &setup_data);
+    int AnalyzeDragonFlyPlusTopo(AnalyzeDFSetupData &setup_data, const char* root_guid_file);
+    int DragonFlyPlusMarkRanks(AnalyzeDFSetupData &setup_data, uint8_t &max_rank);
+    int DragonFlyPlusMarkIslands(AnalyzeDFSetupData &setup_data);
+
+    /*
+     * used by AnalyzeDragonFlyPlusTopo
+     * Add switches from vector of GUIDs to list of roots.
+     */
+    void AddRootSwitchByGUIDs(cl_u64_vector_t *dfp_potential_roots, uint32_t arr_size);
+
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
-    int MarkLeafsByCasNumber(AnalizeDFSetupData &setup_data,
+    int MarkLeafsByCasNumber(AnalyzeDFSetupData &setup_data,
                              SwDbEntryPrtList &leafs_list);
 
-    int MarkLeafsByGroupsNumber(AnalizeDFSetupData &setup_data,
+    int MarkLeafsByGroupsNumber(AnalyzeDFSetupData &setup_data,
                                 SwDbEntryPrtList &leafs_list);
 
     void SetGroupNumber(ARSWDataBaseEntry* p_sw_entry, u_int16_t group_number);
@@ -650,14 +678,14 @@ private:
     int SetPrevGroupNumber(ARSWDataBaseEntry* p_sw_entry,
                            BoolVec &used_group_numbers);
 
-    int DiscoverGroups(AnalizeDFSetupData &setup_data,
+    int DiscoverGroups(AnalyzeDFSetupData &setup_data,
                        SwDbEntryPrtList &leafs_list,
                        BoolVec &used_group_numbers,
                        int cycle = 1);
 
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup update
+     * Helper function for AnalyzeDragonFlyPlusSetup update
      * m_up_ports and m_down_ports data structures
      *
      * @return non zero on failure
@@ -671,73 +699,73 @@ private:
     void UpdateSmDbSwInfo();
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
     int DiscoverLeaf(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         ARSWDataBaseEntry* p_db_entry);
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
     int DiscoverSpine(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         ARSWDataBaseEntry* p_db_entry);
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
     int SetLeaf(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         SwDbEntryPrtList &leafs_list,
         osm_node_t *p_node, bool add_to_list = true);
 
     int SetLeaf(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         osm_node_t *p_node);
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
     int SetSpine(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         osm_node_t *p_nod);
 
     /*
     int SetSpine(
-        AnalizeDFSetupData &setup_data,
+        AnalyzeDFSetupData &setup_data,
         osm_node_t *p_node,
         osm_physp_t *p_physp,
         bool from_down);
         */
 
     /**
-     * Helper function for AnalizeDragonFlyPlusSetup
+     * Helper function for AnalyzeDragonFlyPlusSetup
      *
      * @return non zero on failure
      */
-    //int HandleLeafsWithoutHosts(AnalizeDFSetupData &setup_data,
+    //int HandleLeafsWithoutHosts(AnalyzeDFSetupData &setup_data,
     //                            int &max_group_number);
 
-    int RootDetectionDragonFlyPlusSetup(AnalizeDFSetupData &setup_data);
+    int RootDetectionDragonFlyPlusSetup(AnalyzeDFSetupData &setup_data);
 
     /**
-     * Analize Dragon Fly Plus Setup
+     * Analyze Dragon Fly Plus Setup
      * Set SW type to LEAF or SPINE
      * Set ports as up or down ports
      * @param [out] setup_data
      *
      * @return non zero on failure
      */
-    int AnalizeDragonFlyPlusSetup(AnalizeDFSetupData &setup_data);
+    int AnalyzeDragonFlyPlusSetup(AnalyzeDFSetupData &setup_data);
 
     /** Helper method for ARCalculatePortGroupsDF*/
     void SavePortGroupsAndDump();
@@ -749,7 +777,23 @@ private:
         bool enforce_complete_route,
         list <ARSWDataBaseEntry*> &switches,
         int plft_id,
-        AnalizeDFSetupData &setup_data);
+        AnalyzeDFSetupData &setup_data);
+
+    uint16_t GetGroupNumberRN(
+	uint16_t sw_lid_num,
+	ARSWDataBaseEntry &sw_db_entry,
+	uint8_t plft_id,
+	AnalyzeDFSetupData &setup_data);
+
+    int AssignPortGroupsDFP(
+	ARSWDataBaseEntry &sw_db_entry,
+	GroupsData &groups_data,
+	uint8_t plft_id);
+
+    int AssignGroupIDPerPlftLeafSwitch(
+	ARSWDataBaseEntry &sw_db_entry,
+	GroupsData &groups_data,
+	uint8_t plft_id);
 
     void PrintPSGroupData(const char * str,PSGroupData &group_data);
 
@@ -788,7 +832,7 @@ private:
             u_int16_t calculated_max_lid,
             int plft_id);
 
-    void ARDumpDFAnalizedSetup();
+    void ARDumpDFAnalyzedSetup();
 
     //num_ports - num of phys ports, ports number not including port 0
     u_int8_t GetNextStaticPort(
@@ -800,7 +844,7 @@ private:
     /**
      * Calculate adaptive routing settings according to Dragon Flay algorithm
      */
-    int ARCalculatePortGroupsDF(AnalizeDFSetupData &setup_data);
+    int ARCalculatePortGroupsDF(AnalyzeDFSetupData &setup_data);
 
     /**Set Private LFTs and VL2VL Mapping on the switch*/
     int ARMapPLFTsAndSetSL2VLAct();
@@ -838,7 +882,7 @@ private:
     /*
      * Validate switches capabilities.
      */
-    int ValidateSwitchesCapabilites();
+    int ValidateSwitchesCapabilities();
 
     /**
      * Get SL to VL mapping for certain port on DFP setup
@@ -851,7 +895,6 @@ private:
     int GetSL2VLDF(osm_physp_t *osm_phys_port, uint8_t iport_num, uint8_t oport_num,
 		   uint8_t (*sl2vl)[IB_NUMBER_OF_SLS]);
 
-
     friend class OSM::AdaptiveRoutingAlgorithm;
     friend class OSM::PlftBasedArAlgorithm;
     friend class OSM::ArKdorAlgorithm;
