@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -78,7 +78,7 @@ BEGIN_C_DECLS
 *	OSM_DEFAULT_M_KEY
 *
 * DESCRIPTION
-*	Managment key value used by the OpenSM.
+*	Management key value used by the OpenSM.
 *
 * SYNOPSIS
 */
@@ -100,7 +100,7 @@ BEGIN_C_DECLS
 *	OSM_DEFAULT_SA_KEY
 *
 * DESCRIPTION
-*	Subnet Adminstration key value.
+*	Subnet Administration key value.
 *
 * SYNOPSIS
 */
@@ -128,7 +128,7 @@ BEGIN_C_DECLS
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_MAX_OP_VLS 3
+#define OSM_DEFAULT_MAX_OP_VLS 2
 /********/
 /****d* OpenSM: OSM_DEFAULT_SL
 * NAME
@@ -161,7 +161,7 @@ BEGIN_C_DECLS
 * 	heavy sweep cycle. 0 is never skip the rediscovery loop.
 * SYNOPSIS
 */
-#define OSM_DEFAULT_MAX_SEQ_REDISC 0
+#define OSM_DEFAULT_MAX_SEQ_REDISC 4
 /********/
 /****d* OpenSM: OSM_DEFAULT_SM_PRIORITY
 * NAME
@@ -214,6 +214,8 @@ BEGIN_C_DECLS
 */
 #ifdef __WIN__
 #define OSM_DEFAULT_CACHE_DIR "%TEMP%"
+#elif defined (OPENSM_CACHE_DIR)
+#define OSM_DEFAULT_CACHE_DIR OPENSM_CACHE_DIR
 #else
 #define OSM_DEFAULT_CACHE_DIR "/var/cache/opensm"
 #endif
@@ -306,6 +308,24 @@ BEGIN_C_DECLS
 #endif
 /***********/
 
+/****d* OpenSM: OSM_DEFAULT_PPCC_ALGO_DIR
+* NAME
+*	OSM_DEFAULT_PPCC_ALGO_DIR
+*
+* DESCRIPTION
+*	Specifies the default PPCC algoritns dirp name
+*
+* SYNOPSIS
+*/
+#if defined(HAVE_DEFAULT_PPCC_ALGO_DIR)
+#define OSM_DEFAULT_PPCC_ALGO_DIR HAVE_DEFAULT_PPCC_ALGO_DIR
+#elif defined(OPENSM_CONFIG_DIR)
+#define OSM_DEFAULT_PPCC_ALGO_DIR OPENSM_CONFIG_DIR "/ppcc_algo_dir"
+#else
+#define OSM_DEFAULT_PPCC_ALGO_DIR "/etc/opensm/ppcc_algo_dir"
+#endif
+/***********/
+
 /****d* OpenSM: OSM_DEFAULT_TORUS_CONF_FILE
 * NAME
 *	OSM_DEFAULT_TORUS_CONF_FILE
@@ -378,6 +398,60 @@ BEGIN_C_DECLS
 #endif
 /***********/
 
+/****d* OpenSM: OSM_DEFAULT_FAST_RECOVERY_CONF_FILE
+* NAME
+*	OSM_DEFAULT_FAST_RECOVERY_CONF_FILE
+*
+* DESCRIPTION
+*	Specifies the default fast recovery configuration file name
+*
+* SYNOPSIS
+*/
+#if defined(HAVE_DEFAULT_FAST_RECOVERY_CONF_FILE)
+#define OSM_DEFAULT_FAST_RECOVERY_CONF_FILE HAVE_DEFAULT_FAST_RECOVERY_CONF_FILE
+#elif defined(OPENSM_CONFIG_DIR)
+#define OSM_DEFAULT_FAST_RECOVERY_CONF_FILE OPENSM_CONFIG_DIR "/fast_recovery.conf"
+#else
+#define OSM_DEFAULT_FAST_RECOVERY_CONF_FILE "/etc/opensm/fast_recovery.conf"
+#endif
+/***********/
+
+/****d* OpenSM: OSM_DEFAULT_TENANTS_POLICY_CONF_FILE
+* NAME
+*     OSM_DEFAULT_TENANTS_POLICY_CONF_FILE
+*
+* DESCRIPTION
+*     Specifies the default TENANTS configuration file name
+*
+* SYNOPSIS
+*/
+#if defined(HAVE_OSM_DEFAULT_TENANTS_POLICY_CONF_FILE)
+#define OSM_DEFAULT_TENANTS_POLICY_CONF_FILE HAVE_OSM_DEFAULT_TENANTS_POLICY_CONF_FILE
+#elif defined(OPENSM_CONFIG_DIR)
+#define OSM_DEFAULT_TENANTS_POLICY_CONF_FILE OPENSM_CONFIG_DIR "/tenants-policy.conf"
+#else
+#define OSM_DEFAULT_TENANTS_POLICY_CONF_FILE "/etc/opensm/tenants-policy.conf"
+#endif
+/***********/
+
+/****d* OpenSM: OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE
+* NAME
+*     OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE
+*
+* DESCRIPTION
+*     Specifies the default FABRIC_MODE configuration file name
+*
+* SYNOPSIS
+*/
+#if defined(HAVE_OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE)
+#define OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE HAVE_OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE
+#elif defined(OPENSM_CONFIG_DIR)
+#define OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE OPENSM_CONFIG_DIR "/fabric-mode-policy.conf"
+#else
+#define OSM_DEFAULT_FABRIC_MODE_POLICY_CONF_FILE "/etc/opensm/fabric-mode-policy.conf"
+#endif
+/***********/
+
 /****d* OpenSM: OSM_DEFAULT_PID_FILE
 * NAME
 *	OSM_DEFAULT_PID_FILE
@@ -417,7 +491,7 @@ BEGIN_C_DECLS
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_FTREE_CA_ORDER_DUMP_FILE "opensm-ftree-ca-order"
+#define OSM_DEFAULT_FTREE_CA_ORDER_DUMP_FILE OSM_DUMP_FILES_PREFIX "-ftree-ca-order"
 /***********/
 /****d* OpenSM: OSM_DEFAULT_STATISTICS_DUMP_FILE
 * NAME
@@ -428,7 +502,7 @@ BEGIN_C_DECLS
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_STATISTICS_DUMP_FILE "opensm-statistics"
+#define OSM_DEFAULT_STATISTICS_DUMP_FILE OSM_DUMP_FILES_PREFIX "-statistics"
 /***********/
 /****d* OpenSM: OSM_DEFAULT_SWEEP_INTERVAL_SECS
 * NAME
@@ -461,7 +535,7 @@ BEGIN_C_DECLS
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_LONG_TRANS_TIMEOUT_MILLISEC 500
+#define OSM_DEFAULT_LONG_TRANS_TIMEOUT_MILLISEC 1000
 /***********/
 /****d* OpenSM: OSM_DEFAULT_SUBNET_TIMEOUT
 * NAME
@@ -568,7 +642,6 @@ BEGIN_C_DECLS
 * SYNOPSIS
 */
 #define OSM_DEFAULT_TRAP_SUPPRESSION_TIMEOUT 5000000
-#define OSM_DEFAULT_TRAP_SUPRESSION_TIMEOUT OSM_DEFAULT_TRAP_SUPPRESSION_TIMEOUT
 /***********/
 /****d* OpenSM: OSM_DEFAULT_UNHEALTHY_TIMEOUT
 * NAME
@@ -604,7 +677,7 @@ BEGIN_C_DECLS
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_SMP_MAX_ON_WIRE 16
+#define OSM_DEFAULT_SMP_MAX_ON_WIRE 32
 /***********/
 /****d* OpenSM: OSM_SM_DEFAULT_QP0_RCV_SIZE
 * NAME
@@ -675,7 +748,7 @@ BEGIN_C_DECLS
 *   OSM_SM_DEFAULT_POLLING_TIMEOUT_MILLISECS
 *
 * DESCRIPTION
-*   Specifies the polling timeout (in miliseconds) - the timeout
+*   Specifies the polling timeout (in milliseconds) - the timeout
 *   between one poll to another.
 *
 * SYNOPSIS
@@ -1105,7 +1178,7 @@ typedef enum _osm_thread_state {
 * NAME
 *
 * DESCRIPTION
-*	Defaut SL for pFRN commmunication
+*	Default SL for pFRN communication
 *
 * SYNOPSIS
 */
@@ -1116,7 +1189,7 @@ typedef enum _osm_thread_state {
 * NAME
 *
 * DESCRIPTION
-*	Defaut time (in seconds) since latest pFRN for a specific sub group
+*	Default time (in seconds) since latest pFRN for a specific sub group
 *	was received, after which the entire mask must be cleared
 *
 * SYNOPSIS
@@ -1245,16 +1318,16 @@ typedef enum _osm_thread_state {
 #define OSM_CAP2_IS_QOS_SUPPORTED (1 << 1)
 /***********/
 
-/****d* OpenSM: OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPPORTED
+/****d* OpenSM: OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPORTED
 * Name
-*	OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPPORTED
+*	OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPORTED
 *
 * DESCRIPTION
 *	Reverse path PKeys indicate in PathRecord responses
 *
 * SYNOPSIS
 */
-#define OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPPORTED (1 << 2)
+#define OSM_CAP2_IS_REVERSE_PATH_PKEY_SUPPORTED (1 << 2)
 /***********/
 
 /****d* OpenSM: OSM_CAP2_IS_MCAST_TOP_SUPPORTED
@@ -1427,6 +1500,28 @@ typedef enum _osm_thread_state {
  */
 #define OSM_CAP2_IS_LINK_SPEED_NDR_SUPPORTED (1 << 17)
 
+/****d* OpenSM: OSM_CAP2_IS_EXTENDED_SPEEDS2_SUPPORTED
+ * Name
+ *	OSM_CAP2_IS_EXTENDED_SPEEDS2_SUPPORTED
+ *
+ * DESCRIPTION
+ *	Extended Link Speeds 2 supported
+ *
+ * SYNOPSIS
+ */
+#define OSM_CAP2_IS_EXTENDED_SPEEDS2_SUPPORTED (1 << 19)
+
+/****d* OpenSM: OSM_CAP2_IS_LINK_SPEED_XDR_SUPPORTED
+ * Name
+ *	OSM_CAP2_IS_LINK_SPEED_XDR_SUPPORTED
+ *
+ * DESCRIPTION
+ *	XDR link speed supported
+ *
+ * SYNOPSIS
+ */
+#define OSM_CAP2_IS_LINK_SPEED_XDR_SUPPORTED (1 << 20)
+
 /****d* OpenSM: osm_signal_t
 * NAME
 *	osm_signal_t
@@ -1439,16 +1534,19 @@ typedef enum _osm_thread_state {
 *
 * SYNOPSIS
 */
-#define OSM_SIGNAL_NONE				0
-#define OSM_SIGNAL_SWEEP			1
-#define OSM_SIGNAL_IDLE_TIME_PROCESS_REQUEST	2
-#define OSM_SIGNAL_PERFMGR_SWEEP		3
-#define OSM_SIGNAL_GUID_PROCESS_REQUEST		4
-#define OSM_SIGNAL_VIRTMGR_PROCESS_REQUEST	5
-#define OSM_SIGNAL_INFORM_PROCESS_REQUEST	6
-#define OSM_SIGNAL_DUMP_STATISTIC		7
-#define OSM_SIGNAL_IDLE_TIME_REBALANCE_ROUTING  8
-#define OSM_SIGNAL_MAX				9
+#define OSM_SIGNAL_NONE				        0
+#define OSM_SIGNAL_SWEEP			        1
+#define OSM_SIGNAL_IDLE_TIME_PROCESS_REQUEST	        2
+#define OSM_SIGNAL_PERFMGR_SWEEP		        3
+#define OSM_SIGNAL_GUID_PROCESS_REQUEST		        4
+#define OSM_SIGNAL_VIRTMGR_PROCESS_REQUEST	        5
+#define OSM_SIGNAL_INFORM_PROCESS_REQUEST	        6
+#define OSM_SIGNAL_DUMP_STATISTIC		        7
+#define OSM_SIGNAL_IDLE_TIME_REBALANCE_ROUTING          8
+#define OSM_SIGNAL_FLID_ROUTING_SWEEP		        9
+#define OSM_SIGNAL_PKEY_SWEEP_REQUEST                   10
+#define OSM_SIGNAL_IDLE_TIME_EXTERNAL_PROCESS_REQUEST	11
+#define OSM_SIGNAL_MAX				        12
 
 typedef unsigned int osm_signal_t;
 /***********/
@@ -1563,28 +1661,54 @@ typedef enum _osm_rtr_aguid_mode {
 } osm_rtr_aguid_mode_t;
 /**********/
 
+/****d* OpenSM: OSM_RTR_PR_MTU_USE_MTU_ON_PATH
+* NAME
+*	OSM_RTR_PR_MTU_USE_MTU_ON_PATH
+*
+* DESCRIPTION
+*	A special value for mtu configuration for taking maximal common available MTU
+*	along the path in the path record query from source to destination.
+*
+* SYNOPSIS
+*/
+#define OSM_RTR_PR_MTU_USE_MTU_ON_PATH	255
+
 /****d* OpenSM: MC Member Record Receiver/OSM_DEFAULT_RTR_PR_MTU
 * Name
 *	OSM_DEFAULT_RTR_PR_MTU
 *
 * DESCRIPTION
-*	Default inter subnet PathRecord MTU is <= 2KB
+*	Default inter subnet PathRecord MTU is a maximal common available MTU
+*	on the path from source to the router, that leads to destination on another subnet.
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_RTR_PR_MTU IB_MTU_LEN_2048
+#define OSM_DEFAULT_RTR_PR_MTU OSM_RTR_PR_MTU_USE_MTU_ON_PATH
 /***********/
 
+/****d* OpenSM: OSM_RTR_PR_RATE_USE_RATE_ON_PATH
+* NAME
+*	OSM_RTR_PR_RATE_USE_RATE_ON_PATH
+*
+* DESCRIPTION
+*	A special value for rate configuration for taking maximal common available rate
+*	along the path in the path record query from source to destination.
+*
+* SYNOPSIS
+*/
+#define OSM_RTR_PR_RATE_USE_RATE_ON_PATH	255
+
 /****d* OpenSM: MC Member Record Receiver/OSM_DEFAULT_RTR_PR_RATE
 * Name
 *	OSM_DEFAULT_RTR_PR_RATE
 *
 * DESCRIPTION
-*	Default inter subnet PathRecord Rate is <= EDR
+*	Default inter subnet PathRecord Rate is a maximal common available rate
+*	on the path from source to the router, that leads to destination on another subnet.
 *
 * SYNOPSIS
 */
-#define OSM_DEFAULT_RTR_PR_RATE IB_PATH_RECORD_RATE_100_GBS
+#define OSM_DEFAULT_RTR_PR_RATE OSM_RTR_PR_RATE_USE_RATE_ON_PATH
 /***********/
 /****d* OpenSM: OSM_DEFAULT_RTR_PR_FLOWLABEL
 * NAME
@@ -1613,7 +1737,7 @@ typedef enum _osm_rtr_aguid_mode {
 * 	OSM_DEFAULT_MAX_CAS_ON_SPINE
 * DESCRIPTION
 * 	Default maximum number of CAs on switch, for DF+ routing algorithm
-* 	to consider the swtich as spine.
+* 	to consider the switch as spine.
 * SYNOPSIS
 */
 #define OSM_DEFAULT_MAX_CAS_ON_SPINE 2
@@ -1646,6 +1770,19 @@ typedef enum _osm_rtr_aguid_mode {
 #define OSM_DEFAULT_ADAPTIVE_TIMEOUT_SL_MASK 0xFFFF
 /********/
 
+/****d* OpenSM: OSM_DEFAULT_SUPPRESS_MC_PKEY_TRAPS
+* NAME
+* 	OSM_DEFAULT_SUPPRESS_MC_PKEY_TRAPS
+*
+* DESCRIPTION
+* 	The default parameter for multicast partition key trap suppression.
+* 	When bit 0 is activated these traps will be suppressed.
+* SYNOPSIS
+*/
+#define OSM_DEFAULT_SUPPRESS_MC_PKEY_TRAPS 0x01
+/********/
+
+
 /****d* OpenSM: osm_sm_standalone_t
 * NAME
 *       osm_sm_standalone_t
@@ -1662,5 +1799,30 @@ typedef enum _osm_sm_standalone_state {
 	OSM_STANDALONE_MAX
 } osm_sm_standalone_t;
 
+/****d* OpenSM: OSM_DEFAULT_FLID_COMPRESSION
+* NAME
+* 	OSM_DEFAULT_FLID_COMPRESSION
+*
+* DESCRIPTION
+* 	The default parameter for FLID compression ratio - a maximal number of leaf switches
+* 	that can share the same FLID.
+* SYNOPSIS
+*/
+#define OSM_DEFAULT_FLID_COMPRESSION	2
+/********/
+
+/* Some hardcoded values for Single Node (SN) case of NVLink */
+#define OSM_NVLINK_MIN_PLANES			1
+#define OSM_NVLINK_MAX_PLANES			18
+
+#define OSM_FABRIC_MODE_PROFILE_IGNORE		"none"
+#define OSM_DEFAULT_FABRIC_MODE_PROFILE		OSM_FABRIC_MODE_PROFILE_IGNORE
+
+/* HOQ life time is 5bits field, hence any larger number can be used to mark invalid value */
+#define OSM_HOQ_LIFE_MAX_VALUE			0x1F
+#define OSM_HOQ_LIFE_INVALID			OSM_HOQ_LIFE_MAX_VALUE + 1
+
+#define OSM_STO_MILLISEC			500
+
 END_C_DECLS
 #endif				/* _OSM_BASE_H_ */
