@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2011 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -42,6 +42,7 @@
 #include <complib/cl_dispatcher.h>
 #include <opensm/osm_base.h>
 #include <opensm/osm_log.h>
+#include <opensm/osm_node.h>
 #include <opensm/osm_msgdef.h>
 #include <opensm/osm_path.h>
 
@@ -633,17 +634,19 @@ void osm_dump_smp_dr_path_v2(IN osm_log_t * p_log, IN const ib_smp_t * p_smp,
 			     IN const int file_id, IN osm_log_level_t level);
 
 void osm_dump_dr_smp(IN osm_log_t * p_log, IN const ib_smp_t * p_smp,
-		     IN osm_log_level_t level);
+		     IN ib_net64_t port_guid, IN ib_net64_t dest_guid,
+		     IN osm_log_level_t log_level);
 
 void osm_dump_dr_smp_v2(IN osm_log_t * p_log, IN const ib_smp_t * p_smp,
-			IN ib_net64_t port_guid,
+			IN ib_net64_t port_guid, IN ib_net64_t dest_guid,
 			IN const int file_id, IN osm_log_level_t level);
 
 void osm_dump_sa_mad(IN osm_log_t * p_log, IN const ib_sa_mad_t * p_smp,
 		     IN osm_log_level_t level);
 
 void osm_dump_sa_mad_v2(IN osm_log_t * p_log, IN const ib_sa_mad_t * p_smp,
-			IN const int file_id, IN osm_log_level_t level);
+			IN const int file_id, IN osm_log_level_t level,
+			IN ib_net64_t dest_guid);
 
 void osm_dump_dr_path_as_buf(IN size_t max_len, IN const osm_dr_path_t * p_path,
 			     OUT char* buf);
@@ -824,14 +827,14 @@ const char *osm_get_port_state_str_fixed_width(IN uint8_t port_state);
 
 const char *osm_get_node_type_str_fixed_width(IN uint8_t node_type);
 
-const char *osm_get_manufacturer_str(IN uint64_t guid_ho);
+const char *osm_get_manufacturer_str(IN osm_node_t *p_node);
 
 const char *osm_get_mtu_str(IN uint8_t mtu);
 
 const char *osm_get_lwa_str(IN uint8_t lwa);
 
-const char *osm_get_lsa_str(IN uint8_t lsa, IN uint8_t lsea, IN uint8_t state,
-			    IN uint8_t mepi_active_speed);
+const char *osm_get_lsa_str(IN uint8_t lsa, IN uint8_t lsea, IN uint8_t lsea2,
+			    IN uint8_t state, IN uint8_t mepi_active_speed);
 
 /****f* IBA Base: Types/ib_get_path_record_rate_str
 * NAME
@@ -1063,7 +1066,7 @@ int ib_path_get_reduced_rate(IN const uint8_t rate, IN const uint8_t limit);
 *       osm_truncate_file
 *
 * DESCRIPTION
-*       Truncate a file and reseting the file size to 0.
+*       Truncate a file and resetting the file size to 0.
 *
 * SYNOPSIS
 */
@@ -1176,8 +1179,9 @@ int ib_path_rate_max_12xedr(IN const int rate);
 *	ib_path_rate_2x_hdr_fixups
 *
 * DESCRIPTION
-*	Fixes encoded rate based on whether 2x link width
-*	and/or HDR are supported.
+*	Lowers encoded rate to one that's possible according to the PortInfo's
+*	capability's mask. For example, whether 2x link width and/or HDR are
+*	supported.
 *
 * SYNOPSIS
 */
@@ -1196,6 +1200,10 @@ int ib_path_rate_2x_hdr_fixups(IN const ib_port_info_t * p_pi,
 *	based on whether 2x link width and/or HDR are supported.
 *
 * NOTES
+*	Named ib_path_rate_2x_hdr_fixups since HDR using 2x is the first speed
+*	which had specific capabilitiy bits.
+*	Since EDR doesn't have corresponding capability mask bit, it is assumed
+*	that EDR is supported and can be used as fallback.
 *
 * SEE ALSO
 *********/
@@ -1359,5 +1367,91 @@ void osm_dump_key_violation(IN osm_log_t * p_log,
 * SEE ALSO
 *********/
 
+/****f* OpenSM: Helper/osm_compare_uint64
+* NAME
+*	osm_compare_uint64
+*
+* DESCRIPTION
+*	Compares given values as uint64.
+*
+* SYNOPSIS
+*/
+static inline int osm_compare_uint64(IN const void * p_val_1, IN const void * p_val_2)
+{
+	return memcmp(p_val_1, p_val_2, sizeof(uint64_t));
+}
+/*
+* PARAMETERS
+*	p_val_1
+*		[in] Pointer to the first value for comparison
+*
+*	p_val_2
+*		[in] Pointer to the second value for comparison
+*
+* RETURN VALUES
+*	Returns 0 if the keys match.
+*	Returns less than 0 if *p_key1 is less than *p_key2.
+*	Returns greater than 0 if *p_key1 is greater than *p_key2.
+*
+* NOTES
+*
+* SEE ALSO
+*********/
+
+/****f* OpenSM: Helper/osm_compare_uint16
+* NAME
+*	osm_compare_uint16
+*
+* DESCRIPTION
+*	Compares given values as uint16.
+*
+* SYNOPSIS
+*/
+static inline int osm_compare_uint16(IN const void * p_val_1, IN const void * p_val_2)
+{
+	return memcmp(p_val_1, p_val_2, sizeof(uint16_t));
+}
+/*
+* PARAMETERS
+*	p_val_1
+*		[in] Pointer to the first value for comparison
+*
+*	p_val_2
+*		[in] Pointer to the second value for comparison
+*
+* RETURN VALUES
+*	Returns 0 if the keys match.
+*	Returns less than 0 if *p_key1 is less than *p_key2.
+*	Returns greater than 0 if *p_key1 is greater than *p_key2.
+*
+* NOTES
+*
+* SEE ALSO
+*********/
+
+/****f* OpenSM: Helper/osm_get_number_of_set_bits_in_bitmask
+* NAME
+*	osm_get_number_of_set_bits_in_bitmask
+*
+* DESCRIPTION
+*	Counts and returns number of set bits in the number
+*
+* SYNOPSIS
+*/
+uint8_t osm_get_number_of_set_bits_in_bitmask(IN int bitmask);
+/*
+* PARAMETERS
+*	bitmask
+*		[in] A bit mask/numeric value to count number of set bits in
+*
+*
+* RETURN VALUES
+* 	Returns number of set bits in the given bit mask.
+*
+* NOTES
+*
+* SEE ALSO
+*********/
+
 END_C_DECLS
 #endif				/* _OSM_HELPER_H_ */
