@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2023-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2012 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -118,7 +119,7 @@ typedef uint8_t osm_log_level_t;
 /*
 	DEFAULT - turn on ERROR and INFO only
 */
-#define OSM_LOG_DEFAULT_LEVEL		OSM_LOG_ERROR | OSM_LOG_INFO
+#define OSM_LOG_DEFAULT_LEVEL		(OSM_LOG_ERROR | OSM_LOG_INFO)
 #define OSM_SYSLOG_LOG_DEFAULT_LEVEL	OSM_LOG_SYS
 
 /****s* OpenSM: Log/osm_log_t
@@ -607,64 +608,6 @@ extern void osm_log_raw(IN osm_log_t * p_log, IN osm_log_level_t verbosity,
 	} while (0)
 
 
-#define DBG_CL_LOCK 0
-
-#define CL_PLOCK_EXCL_ACQUIRE( __exp__ )  \
-{											    		\
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_excl_acquire: Acquiring %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-   cl_plock_excl_acquire( __exp__ );      \
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_excl_acquire: Acquired %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-}
-
-#define CL_PLOCK_ACQUIRE( __exp__ )  \
-{											    		\
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_acquire: Acquiring %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-   cl_plock_acquire( __exp__ );      \
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_acquire: Acquired %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-}
-
-#define CL_PLOCK_RELEASE( __exp__ )  \
-{											    		\
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_release: Releasing %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-   cl_plock_release( __exp__ );      \
-   if (DBG_CL_LOCK)                      \
-     printf("cl_plock_release: Released  %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-}
-
-#define DBG_CL_SPINLOCK 0
-#define CL_SPINLOCK_RELEASE( __exp__ )  \
-{											    		\
-   if (DBG_CL_SPINLOCK)                      \
-     printf("cl_spinlock_release: Releasing %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-   cl_spinlock_release( __exp__ );      \
-   if (DBG_CL_SPINLOCK)                      \
-     printf("cl_spinlock_release: Released  %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-}
-
-#define CL_SPINLOCK_ACQUIRE( __exp__ )  \
-{											    		\
-   if (DBG_CL_SPINLOCK)                      \
-     printf("cl_spinlock_acquire: Acquiring %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-   cl_spinlock_acquire( __exp__ );      \
-   if (DBG_CL_SPINLOCK)                      \
-     printf("cl_spinlock_acquire: Acquired %p file %s, line %d\n", \
-          __exp__,__FILE__, __LINE__);            \
-}
-
 /****f* OpenSM: Helper/osm_is_debug
 * NAME
 *	osm_is_debug
