@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2005 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -112,12 +112,15 @@ typedef enum osm_ignore_port_t {
 	OSM_IGNORE_PORT_NONE = 0,
 	OSM_IGNORE_PORT_ISOLATE,
 	OSM_IGNORE_PORT_HELDBACK,
-	OSM_IGNORE_PORT_NO_DISCOVER
+	OSM_IGNORE_PORT_NO_DISCOVER,
+	OSM_IGNORE_PORT_TOPOCONFIG_MISMATCH
 } osm_ignore_port_t;
 /***********/
 
 typedef struct osm_conf_params_node {
 	uint8_t sharp_enabled;
+	ib_qos_config_vl_t qos_config_vl;
+	boolean_t is_qos_config_vl_parsed;
 } osm_conf_params_node_t;
 
 typedef struct osm_n2n_node {
@@ -140,7 +143,7 @@ typedef struct osm_n2n_node {
 typedef struct osm_node {
 	cl_map_item_t map_item;
 	struct osm_switch *sw;
-	struct osm_router *router;
+	struct osm_gpu *p_gpu;
 	ib_node_info_t node_info;
 	ib_node_desc_t node_desc;
 	uint32_t discovery_count;
@@ -158,6 +161,10 @@ typedef struct osm_node {
 	osm_conf_params_node_t conf_params;
 	boolean_t force_drop_node;
 	osm_n2n_node_data_t n2n_data;
+	uint64_t state_timestamp_microsec;
+	osm_node_hierarchy_info_t hierarchy_info;
+	ib_chassis_info_t chassis_info;
+	boolean_t chassis_info_updated;
 	osm_physp_t physp_table[1];
 } osm_node_t;
 /*
@@ -170,11 +177,6 @@ typedef struct osm_node {
 *		structure. NULL for non-switch nodes. Can be used for fast
 *		access to switch object and for simple node type detection
 *
-* 	router
-*		For router node contains pointer to appropriate osm_router
-*		structure. NULL for non-router nodes.  Can be used for fast
-*		access to router object and for simple node type detection
-*
 *	node_info
 *		The IBA defined NodeInfo data for this node.
 *
@@ -189,7 +191,7 @@ typedef struct osm_node {
 *	phsyp_tbl_size
 *		The size of the physp_table array.  This value is one greater
 *		than the number of ports in the node, since port numbers
-*		start with 1 for some bizzare reason.
+*		start with 1 for some bizarre reason.
 *
 *	print_desc
 *		A printable version of the node description.
@@ -237,6 +239,19 @@ typedef struct osm_node {
 *		Structure used for saving latest NeighborInfo data that was configured on
 *		this node, in order to check if should resent on next calculation.
 *
+*	state_timestamp_microsec
+*	        Timestamp of the last time of the ports changed its state
+*
+*	hierarchy_info
+*		The node data from hierarchy information defined in IBA.
+*
+*	chassis_info
+*		ChassisInfo data for this node.
+*
+*	chassis_info_updated
+*		Indicator, indicates that the node has updated copy of
+*		chassis_info.
+*
 *	phsyp_table
 *		Array of physical port objects belonging to this node.
 *		Index is contiguous by local port number.
@@ -305,6 +320,36 @@ osm_node_t *osm_node_new(IN const osm_madw_t * p_madw, uint8_t sm_port_idx);
 *	Node object
 *********/
 
+/****f* OpenSM: Node/osm_node_get_remote_ignore_type
+* NAME
+*	osm_node_get_remote_ignore_type
+*
+* DESCRIPTION
+*	Returns the ignore type of the node at the relevant port.
+*
+* SYNOPSIS
+*/
+static inline osm_ignore_port_t osm_node_get_remote_ignore_type(IN osm_node_t *p_node,
+								IN const uint8_t port_num)
+{
+	return p_node->remote_ignore_type[port_num];
+}
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*	port_num
+*		[in] Port number of remote ignore type.
+*
+* RETURN VALUES
+*	The ignore type of the port on the remote side of the node at the port number.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
 /****f* OpenSM: Node/osm_node_get_physp_ptr
 * NAME
 *	osm_node_get_physp_ptr
@@ -344,6 +389,34 @@ static inline osm_physp_t *osm_node_get_physp_ptr(IN osm_node_t * p_node,
 *	Node object
 *********/
 
+/****f* OpenSM: Node/osm_node_is_switch
+* NAME
+*	osm_node_is_switch
+*
+* DESCRIPTION
+*	Returns whether the node's IBA type is IB_NODE_TYPE_SWITCH.
+*
+* SYNOPSIS
+*/
+static inline boolean_t osm_node_is_switch(IN const osm_node_t * p_node)
+{
+	return (IB_NODE_TYPE_SWITCH == p_node->node_info.node_type);
+}
+
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+* RETURN VALUES
+*	Returns TRUE if the node is a switch and FALSE otherwise.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
 /****f* OpenSM: Node/osm_node_get_type
 * NAME
 *	osm_node_get_type
@@ -441,6 +514,37 @@ osm_node_t *osm_node_get_remote_node(IN osm_node_t * p_node,
 *	Node object
 *********/
 
+/****f* OpenSM: Node/osm_node_is_remote_switch
+* NAME
+*	osm_node_is_remote_switch
+*
+* DESCRIPTION
+*	Returns TRUE if the remote node connected to the 
+*	other end of the specified port is a switch node.
+*	Returns FALSE if the remote doesn't exist or if it's not a switch.
+*
+* SYNOPSIS
+*/
+boolean_t osm_node_is_remote_switch(IN osm_node_t * p_node, IN uint8_t port_num);
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+*	port_num
+*		[in] Port number in p_node through which to get the remote node.
+*
+* RETURN VALUES
+*	Returns TRUE if the remote node connected to the 
+*	other end of the specified port is a switch node.
+*	Returns FALSE if the remote doesn't exist or if it's not a switch.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
 /****f* OpenSM: Node/osm_node_get_base_lid
 * NAME
 *	osm_node_get_base_lid
@@ -476,6 +580,38 @@ static inline ib_net16_t osm_node_get_base_lid(IN const osm_node_t * p_node,
 *	Node object
 *********/
 
+/****f* OpenSM: Node/osm_node_get_base_lid_ho
+* NAME
+*	osm_node_get_base_lid_ho
+*
+* DESCRIPTION
+*	Returns the host order LID value of the specified port on this node.
+*
+* SYNOPSIS
+*/
+static inline uint16_t osm_node_get_base_lid_ho(IN const osm_node_t * p_node,
+						IN uint32_t port_num)
+{
+	return cl_ntoh16(osm_node_get_base_lid(p_node, port_num));
+}
+
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+*	port_num
+*		[in] Local port number.
+*
+* RETURN VALUES
+*	Returns the port lid in host order.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
 /****f* OpenSM: Node/osm_node_get_remote_base_lid
 * NAME
 *	osm_node_get_remote_base_lid
@@ -607,6 +743,34 @@ static inline ib_net64_t osm_node_get_node_guid(IN const osm_node_t * p_node)
 *	Node object
 *********/
 
+/****f* OpenSM: Node/osm_node_get_node_guid_ho
+* NAME
+*	osm_node_get_node_guid_ho
+*
+* DESCRIPTION
+*	Returns the node GUID of this node in host order.
+*
+* SYNOPSIS
+*/
+static inline uint64_t osm_node_get_node_guid_ho(IN const osm_node_t * p_node)
+{
+	return cl_ntoh64(p_node->node_info.node_guid);
+}
+
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+* RETURN VALUES
+*	Returns the node GUID of this node in host order.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
 /****f* OpenSM: Node/osm_node_link
 * NAME
 *	osm_node_link
@@ -910,5 +1074,76 @@ uint64_t osm_node_get_any_topo_id(IN osm_node_t * p_node);
 *	topo_id for first topology found.
 *	NO_TOPO_ID if the node is not member of any topology.
 *********/
+
+/****f* OpenSM: Node/osm_node_get_extended_node_type
+* NAME
+*	osm_node_get_extended_node_type
+*
+* DESCRIPTION
+*	Returns the type of this node.
+*
+* SYNOPSIS
+*/
+uint8_t osm_node_get_extended_node_type(osm_node_t * p_node);
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+* RETURN VALUES
+*	Returns the IBA defined extended node type of this node.
+*
+* NOTES
+*
+* SEE ALSO
+*	Node object
+*********/
+
+/****f* OpenSM: Node/osm_node_is_gpu
+* NAME
+*	osm_node_is_gpu
+*
+* DESCRIPTION
+*	Return whether inpute node is a GPU
+*
+* SYNOPSIS
+*/
+boolean_t osm_node_is_gpu(IN osm_node_t * p_node);
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+* RETURN VALUES
+*	Return whether inpute node is a GPU
+*
+*********/
+
+/****f* OpenSM: Node/osm_node_get_valid_physp
+* NAME
+*	osm_node_get_valid_physp
+*
+* DESCRIPTION
+*	Returns a pointer to one of the valid physical ports of the node.
+*	If there is no such a port, returns NULL.
+*
+* SYNOPSIS
+*/
+osm_physp_t * osm_node_get_valid_physp(IN osm_node_t * p_node);
+/*
+* PARAMETERS
+*	p_node
+*		[in] Pointer to an osm_node_t object.
+*
+* RETURN VALUES
+*	Returns a pointer to one of the valid physical ports of the node.
+*	If there is no such a port, returns NULL.
+*
+* NOTES
+*
+* SEE ALSO
+*	Physical Port object
+*********/
+
 END_C_DECLS
 #endif				/* _OSM_NODE_H_ */
