@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2021-2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2021-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2006 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -93,6 +93,14 @@ BEGIN_C_DECLS
 *
 *********/
 
+typedef struct osm_perflog_mgr {
+	int reports_num;
+	FILE *perflog_dump_file;
+	unsigned long perflog_file_size;
+	unsigned long perflog_max_size;
+	char *current_str;
+} osm_perflog_mgr_t;
+
 typedef enum statistic_counters {
 	HEAVY_SWEEP_COUNT = 0,
 	LIGHT_SWEEP_COUNT,
@@ -107,6 +115,8 @@ typedef enum statistic_counters {
 	NODE_INFO_SEND_COUNT,
 	SWITCH_INFO_RCV_COUNT,
 	SWITCH_INFO_SEND_COUNT,
+	EXTENDED_SWITCH_INFO_RCV_COUNT,
+	EXTENDED_SWITCH_INFO_SEND_COUNT,
 	GUID_INFO_RCV_COUNT,
 	GUID_INFO_SEND_COUNT,
 	PORT_INFO_RCV_COUNT,
@@ -153,6 +163,10 @@ typedef enum statistic_counters {
 	RN_GEN_BY_SUB_GROUP_PRIORITY_SEND_COUNT,
 	MFT_RCV_COUNT,
 	MFT_SEND_COUNT,
+	HIERARCHY_INFO_RCV_COUNT,
+	HIERARCHY_INFO_SEND_COUNT,
+	CHASSIS_INFO_RCV_COUNT,
+	CHASSIS_INFO_SEND_COUNT,
 	SM_INFO_RCV_COUNT,
 	SM_INFO_SEND_COUNT,
 	MEPI_RCV_COUNT,
@@ -163,8 +177,16 @@ typedef enum statistic_counters {
 	RTR_ADJ_SITE_LOCAL_TBL_SEND_COUNT,
 	RTR_NEXT_HOP_TBL_RCV_COUNT,
 	RTR_NEXT_HOP_TBL_SEND_COUNT,
+	RTR_ADJ_ROUTER_LID_INFO_RCV_COUNT,
+	RTR_ADJ_ROUTER_LID_INFO_SEND_COUNT,
+	RTR_ROUTER_LID_TBL_RCV_COUNT,
+	RTR_ROUTER_LID_TBL_SEND_COUNT,
+	RTR_ROUTER_AR_GROUP_TO_LID_TBL_RCV_COUNT,
+	RTR_ROUTER_AR_GROUP_TO_LID_TBL_SEND_COUNT,
 	QOS_CONFIG_SL_RCV_COUNT,
 	QOS_CONFIG_SL_SEND_COUNT,
+	QOS_CONFIG_VL_RCV_COUNT,
+	QOS_CONFIG_VL_SEND_COUNT,
 	VIRTUALIZATION_INFO_RCV_COUNT,
 	VIRTUALIZATION_INFO_SEND_COUNT,
 	VPORT_PORT_STATE_TBL_RCV_COUNT,
@@ -201,6 +223,10 @@ typedef enum statistic_counters {
 	CC_HCA_RP_PARAMETERS_SEND_COUNT,
 	CC_HCA_NP_PARAMETERS_RCV_COUNT,
 	CC_HCA_NP_PARAMETERS_SEND_COUNT,
+	PPCC_HCA_ALGO_CONFIG_RCV_COUNT,
+	PPCC_HCA_ALGO_CONFIG_SEND_COUNT,
+	PPCC_HCA_ALGO_PARAMS_CONFIG_RCV_COUNT,
+	PPCC_HCA_ALGO_PARAMS_CONFIG_SEND_COUNT,
 	N2N_CLASS_PORT_INFO_RCV_COUNT,
 	N2N_CLASS_PORT_INFO_SEND_COUNT,
 	N2N_KEY_INFO_RCV_COUNT,
@@ -268,8 +294,36 @@ typedef enum statistic_counters {
 	HBF_CONFIG_SEND_COUNT,
 	WHBF_CONFIG_RCV_COUNT,
 	WHBF_CONFIG_SEND_COUNT,
+	NVL_HBF_CONFIG_RCV_COUNT,
+	NVL_HBF_CONFIG_SEND_COUNT,
 	PFRN_CONFIG_RCV_COUNT,
 	PFRN_CONFIG_SEND_COUNT,
+	PROFILES_CONFIG_RCV_COUNT,
+	PROFILES_CONFIG_SEND_COUNT,
+	CREDIT_WATCHDOG_CONFIG_RCV_COUNT,
+	CREDIT_WATCHDOG_CONFIG_SEND_COUNT,
+	BER_CONFIG_RCV_COUNT,
+	BER_CONFIG_SEND_COUNT,
+	ALID_INFO_RCV_COUNT,
+	ALID_INFO_SEND_COUNT,
+	RAIL_FILTER_CONFIG_RCV_COUNT,
+	RAIL_FILTER_CONFIG_SEND_COUNT,
+	NOTICE_1601_SEND_COUNT,
+	LFT_SPLIT_RCV_COUNT,
+	LFT_SPLIT_SEND_COUNT,
+	END_PORT_PLANE_FILTER_CONFIG_RCV_COUNT,
+	END_PORT_PLANE_FILTER_CONFIG_SEND_COUNT,
+	NOTICE_1605_SEND_COUNT,
+	NOTICE_1606_SEND_COUNT,
+	NOTICE_1609_SEND_COUNT,
+	NOTICE_1610_SEND_COUNT,
+	CND_INFO_RCV_COUNT,
+	CND_INFO_SEND_COUNT,
+	CND_PORT_STATE_RCV_COUNT,
+	CND_PORT_STATE_SEND_COUNT,
+	CND_PORT_STATE_CHANGE_TRAP_RCV_COUNT, /* notice 1317 */
+	NOTICE_1318_RCV_COUNT,
+	NOTICE_1319_RCV_COUNT,
 	LAST_STAT_COUNTER
 } statistic_counters;
 
@@ -308,7 +362,6 @@ typedef struct osm_opensm {
 	osm_perfmgr_t perfmgr;
 #endif				/* ENABLE_OSM_PERF_MGR */
 	osm_congestion_control_t cc;
-	cl_qlist_t plugin_list;
 	osm_db_t db;
 	boolean_t mad_pool_constructed;
 	osm_mad_pool_t mad_pool;
@@ -334,6 +387,7 @@ typedef struct osm_opensm {
 	unsigned long statistic_file_size[OSM_MAX_BINDING_PORTS + 1];
         unsigned long statistic_max_size;
 	uint64_t stats_counters[OSM_MAX_BINDING_PORTS][OSM_STATISTIC_COUNTERS_SIZE];
+	osm_perflog_mgr_t perflog_mgr;
 	osm_vendor_specific_t vs;
 	char start_time_str[OSM_TIMESTAMP_STR_LEN];
 	osm_n2n_t n2n;
@@ -353,7 +407,7 @@ typedef struct osm_opensm {
 *		The Subnet Administration (SA) object for this subnet.
 *
 *	db
-*		Persistant storage of some data required between sessions.
+*		Persistent storage of some data required between sessions.
 *
 *	mad_pool
 *		Pool of Management Datagram (MAD) objects.
@@ -425,6 +479,25 @@ typedef struct osm_opensm {
 * SEE ALSO
 *********/
 
+/****d* OpenSM: OpenSM/osm_dump_flag_enum
+* NAME
+*	osm_dump_flag_enum
+*
+* DESCRIPTION
+*	Flags of dump file types.
+*
+* SYNOPSIS
+*/
+typedef enum _osm_dump_flag_enum {
+	OSM_DUMP_FLAG_ROUTING =		0x1,
+	OSM_DUMP_FLAG_SUBNET_LST =	0x2,
+	OSM_DUMP_FLAG_VIRTUALIZATION =	0x4,
+	OSM_DUMP_FLAG_ROUTERS =		0x8,
+	OSM_DUMP_FLAG_SMDB =		0x10,
+	OSM_DUMP_FLAG_ALL =		0xFFFF
+} osm_dump_flag_enum;
+/***********/
+
 /****f* OpenSM: OpenSM/osm_opensm_construct
 * NAME
 *	osm_opensm_construct
@@ -459,7 +532,7 @@ void osm_opensm_construct(IN osm_opensm_t * p_osm);
 *
 * DESCRIPTION
 *	The osm_opensm_construct_finish function completes
-*	the second phase of constucting an OpenSM object.
+*	the second phase of constructing an OpenSM object.
 *
 * SYNOPSIS
 */
@@ -601,6 +674,33 @@ ib_api_status_t osm_opensm_init_finish(IN osm_opensm_t * p_osm,
 *	osm_opensm_destroy, osm_opensm_destroy_finish
 *********/
 
+/****f* OpenSM: OpenSM/osm_opensm_set_exit_flag
+* NAME
+*	osm_opensm_set_exit_flag
+*
+* DESCRIPTION
+*	Set SM exit flag to initiate shutdown flow
+*
+* SYNOPSIS
+*/
+void osm_opensm_set_exit_flag(IN osm_opensm_t * p_osm);
+/*
+* PARAMETERS
+*	p_osm
+*		[in] Pointer to an osm_opensm_t object to initialize.
+*
+* RETURN VALUES
+*	IB_SUCCESS if the OpenSM object was initialized successfully.
+*
+* RETURN VALUE
+*	This function does not return a value.
+*
+*
+* SEE ALSO
+*	SM object, osm_opensm_construct, osm_opensm_construct_finish,
+*	osm_opensm_destroy, osm_opensm_destroy_finish
+*********/
+
 /****f* OpenSM: OpenSM/osm_opensm_sweep
 * NAME
 *	osm_opensm_sweep
@@ -736,12 +836,75 @@ void osm_opensm_report_event(osm_opensm_t *osm, osm_epi_event_id_t event_id,
 			     void *event_data);
 
 /* dump helpers */
+
+#define OSM_PERFLOG_EVENT_NAME_LEN	256
+#define OSM_PERFLOG_KEY_INDENT		4
+#define OSM_PERFLOG_MAIN_EVENT		"HEAVYSWEEP"
+
+typedef enum osm_perflog_event_state {
+	OSM_PERFLOG_EVENT_START,
+	OSM_PERFLOG_EVENT_END
+} osm_perflog_event_state_enum;
+
+void osm_perflog(osm_opensm_t *p_osm, const char * name, const char *suffix, osm_perflog_event_state_enum event_state);
+
+#define LOGPERF(p_osm, name, suffix, type) do { \
+		if (p_osm->subn.opt.enable_performance_logging) \
+			osm_perflog(p_osm, name, suffix, type); \
+	} while (0)
+
+#define LOGPERF_START(p_osm, name, suffix) \
+	LOGPERF(p_osm, name, suffix, OSM_PERFLOG_EVENT_START)
+
+#define LOGPERF_END(p_osm, name, suffix) \
+	LOGPERF(p_osm, name, suffix, OSM_PERFLOG_EVENT_END)
+
+/****d* OpenSM: OpenSM/osm_perflog_event_t
+* NAME
+*       osm_perflog_event_t
+*
+* DESCRIPTION
+*	Represents the event data needed for measuring the performance of the event,
+*	and logging to the performance report.
+*
+* SYNOPSIS
+*/
+typedef struct osm_perflog_event {
+	cl_list_item_t list_item;
+	char event_name[OSM_PERFLOG_EVENT_NAME_LEN];
+	struct timeval start_time;
+	int subtasks;
+	int subtask_num;
+	int indent;
+} osm_perflog_event_t;
+/*
+* FIELDS
+*	list_item
+*		Linkage structure for cl_qlist.
+*
+*	event_name
+*		Event name.
+*
+*	start_time
+*		Event start time.
+*
+*	subtasks
+*		Number of subtasks that the event has.
+*
+*	subtask_num
+*		In case the event is a sub task of a bigger task,
+*		subtask_num represents the index of the sub task.
+*
+*	indent
+*		Indention that is added to the event when printed to the file.
+* SEE ALSO
+*********/
+
 void osm_dump_virtualization(osm_opensm_t * osm);
 void osm_dump_mcast_routes(osm_opensm_t * osm);
 void osm_dump_subn_all(osm_opensm_t * osm, osm_subn_t *p_subn, char* prefix,
-		       boolean_t routing_only);
+		       unsigned dump_flags);
 void osm_dump_lfts(osm_opensm_t * osm, osm_subn_t *p_subn, char* prefix);
-void osm_dump_main_all(osm_opensm_t * osm);
 void osm_dump_qmap_to_file(osm_opensm_t * p_osm, const char *file_name,
 			   cl_qmap_t * map,
 			   void (*func) (cl_map_item_t *, FILE *, void *),
@@ -751,6 +914,8 @@ void osm_dump_func_to_file(osm_opensm_t * p_osm, const char *file_path,
 			   void *cxt);
 void osm_dump_statistic(osm_opensm_t * p_osm);
 void osm_dump_sm_db(IN osm_opensm_t * p_osm);
+void osm_dump_heldback_switches(IN osm_opensm_t * p_osm);
+int find_module_name(const char *name, uint8_t *file_id);
 
 /****v* OpenSM/osm_exit_flag
 */
