@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2022 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
+ * Copyright (c) 2022-2024 NVIDIA CORPORATION & AFFILIATES. All rights reserved.
  * Copyright (c) 2004-2009 Voltaire, Inc. All rights reserved.
  * Copyright (c) 2002-2013 Mellanox Technologies LTD. All rights reserved.
  * Copyright (c) 1996-2003 Intel Corporation. All rights reserved.
@@ -74,6 +74,10 @@
 #include <opensm/osm_enhanced_qos_mgr.h>
 #include <opensm/osm_congestion_control.h>
 #include <opensm/osm_key_mgr.h>
+#include <opensm/osm_router_mgr.h>
+#include <opensm/osm_tenant_mgr.h>
+#include <opensm/osm_issu_mgr.h>
+#include <opensm/osm_nvlink.h>
 
 #ifdef __cplusplus
 #  define BEGIN_C_DECLS extern "C" {
@@ -113,6 +117,10 @@ typedef enum osm_crc_id {
 	OSM_CRC_NODE_NAME_MAP_FILE,
 	OSM_CRC_SERVICE_NAME2KEY_MAP_FILE,
 	OSM_CRC_DEVICE_CONFIGURATION_FILE,
+	OSM_CRC_TOPO_CONFIGURATION_FILE,
+	OSM_CRC_FAST_RECOVERY_CONF_FILE,
+	OSM_CRC_TENANTS_POLICY_CONF_FILE,
+	OSM_CRC_FABRIC_MODE_POLICY_CONF_FILE,
 	OSM_CRC_MAX
 } osm_crc_id_enum;
 
@@ -223,6 +231,8 @@ typedef struct osm_sm {
 	cl_disp_reg_handle_t si_disp_h;
 	cl_disp_reg_handle_t lft_disp_h;
 	cl_disp_reg_handle_t mft_disp_h;
+	cl_disp_reg_handle_t hierarchy_disp_h;
+	cl_disp_reg_handle_t chassis_info_disp_h;
 	cl_disp_reg_handle_t sm_info_disp_h;
 	cl_disp_reg_handle_t sm_info_sminfo_get_disp_h;
 	cl_disp_reg_handle_t trap_disp_h;
@@ -238,22 +248,32 @@ typedef struct osm_sm {
 	cl_disp_reg_handle_t vpkey_disp_h;
 	cl_disp_reg_handle_t vnd_disp_h;
 	cl_disp_reg_handle_t ri_disp_h;
-	cl_disp_reg_handle_t adj_rtr_disp_h;
 	cl_disp_reg_handle_t next_hop_rtr_disp_h;
+	cl_disp_reg_handle_t rtr_tbl_mads_disp_h;
 	cl_disp_reg_handle_t qos_config_sl_h;
+	cl_disp_reg_handle_t qos_config_vl_h;
 	cl_disp_reg_handle_t sps_disp_h;
 	cl_disp_reg_handle_t ar_mad_disp_h;
 	cl_disp_reg_handle_t rn_mad_disp_h;
 	cl_disp_reg_handle_t plft_mad_disp_h;
 	cl_disp_reg_handle_t eni_disp_h;
+	cl_disp_reg_handle_t profiles_disp_h;
+	cl_disp_reg_handle_t fast_recovery_disp_h;
+	cl_disp_reg_handle_t rail_filter_config_disp_h;
+	cl_disp_reg_handle_t end_port_plane_filter_config_disp_h;
+	cl_disp_reg_handle_t extended_switch_info_disp_h;
 	boolean_t hm_dirty;
 	osm_virt_mgr_t virt_mgr;
+	osm_tenant_mgr_t tenant_mgr;
 	osm_crc_element_t *files_crc;
 	uint16_t crc_changed;
 	boolean_t sharp_nodes_disabled;
 	osm_router_policy_t router_policy_mgr;
 	osm_log_verbose_bypass_t verbose_bypass;
 	osm_capability_mgr_t cap_mgr;
+	osm_router_mgr_t router_mgr;
+	osm_nvlink_mgr_t nvl_mgr;
+	osm_issu_mgr_t issu_mgr;
 } osm_sm_t;
 /*
 * FIELDS
@@ -303,6 +323,12 @@ typedef struct osm_sm {
 *	cap_mgr
 *		Manager of capabilities
 *
+*	router_mgr
+*		Manager of routers
+*
+*	nvlink_mgr
+*		Manager for NVLink configuration
+*
 * SEE ALSO
 *	SM object
 *********/
@@ -532,7 +558,7 @@ ib_api_status_t osm_sm_bind(IN osm_sm_t * p_sm, IN ib_net64_t * port_guids);
 *
 * DESCRIPTION
 *	Starts the process to transmit a directed route request for
-*	the attribute. The request is sent via less loaded OpenSM port.
+*	the attribute. The request may be sent via less loaded OpenSM port.
 *
 * SYNOPSIS
 */
@@ -597,6 +623,10 @@ ib_api_status_t osm_send_req(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 *	This function asynchronously requests the specified attribute.
 *	The response from the node will be routed through the Dispatcher
 *	to the appropriate receive controller object.
+*	
+*	Also, currently, GET MADs are always sent on the main port.
+*	If we introduce sending on the least loaded port to GET MADs as well,
+*	it will be in a separate commit.
 *********/
 
 /****f* OpenSM: SM/osm_req_get
@@ -610,6 +640,7 @@ ib_api_status_t osm_send_req(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 * SYNOPSIS
 */
 ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
+                            IN const osm_physp_t * p_physp,
 			    IN ib_net16_t attr_id, IN ib_net32_t attr_mod,
 			    IN boolean_t find_mkey, IN ib_net64_t m_key,
 			    IN uint32_t timeout, IN cl_disp_msgid_t err_msg,
@@ -623,6 +654,10 @@ ib_api_status_t osm_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_path,
 *		[in] Pointer to the directed route path to the node
 *		from which to retrieve the attribute.
 *
+*	p_physp
+*		[in] Pointer to osm_physp_t object for taking direct route
+*		of the least loaded OpenSM port.
+*
 *	attr_id
 *		[in] Attribute ID to request.
 *
@@ -668,6 +703,7 @@ osm_madw_t *osm_prepare_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 				IN ib_net16_t attr_id, IN ib_net32_t attr_mod,
 				IN boolean_t find_mkey, IN ib_net64_t m_key,
 				IN uint32_t timeout, IN cl_disp_msgid_t err_msg,
+				IN ib_net64_t dest_guid,
 				IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -696,6 +732,9 @@ osm_madw_t *osm_prepare_req_get(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
+*	dest_guid
+*		[in] Destination node GUID for this MAD.
+*
 *	p_context
 *		[in] Mad wrapper context structure to be copied into the wrapper
 *		     context, and thus visible to the recipient of the response.
@@ -763,6 +802,7 @@ osm_madw_t *osm_prepare_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 				IN ib_net32_t attr_mod, IN boolean_t find_mkey,
 				IN ib_net64_t m_key, IN uint32_t timeout,
 				IN cl_disp_msgid_t err_msg,
+				IN ib_net64_t dest_guid,
 				IN const osm_madw_context_t * p_context);
 /*
 * PARAMETERS
@@ -797,6 +837,9 @@ osm_madw_t *osm_prepare_req_set(IN osm_sm_t * sm, IN const osm_dr_path_t * p_pat
 *	err_msg
 *		[in] Message id with which to post this MAD if an error occurs.
 *
+*	dest_guid
+*		[in] Destination node GUID for this MAD.
+*
 *	p_context
 *		[in] Mad wrapper context structure to be copied into the wrapper
 *		     context, and thus visible to the recipient of the response.
@@ -972,7 +1015,61 @@ ib_api_status_t osm_req_get_pi(IN osm_sm_t * sm, IN const osm_physp_t *p_physp,
 *
 *********/
 
+/****f* OpenSM: SM/osm_req_get_esi
+* NAME
+*	osm_req_get_esi
+*
+* DESCRIPTION
+* 	Get the switch ExtendedSwitchInfo attribute.
+*
+* SYNOPSIS
+*/
+ib_api_status_t osm_req_get_esi(IN osm_sm_t *p_sm,
+				IN osm_switch_t *p_sw);
+/*
+* PARAMETERS
+*	p_sm
+*		[in] Pointer to an osm_sm_t object.
+*
+*	p_sw
+*		[in] Pointer to an osm_switch_t object.
+*
+* RETURN VALUE
+*	ib_api_status_t of the request
+*
+* NOTES
+*
+*********/
 
+/****f* OpenSM: SM/osm_req_set_esi
+* NAME
+*	osm_req_set_esi
+*
+* DESCRIPTION
+* 	Set the switch ExtendedSwitchInfo attribute.
+*
+* SYNOPSIS
+*/
+ib_api_status_t osm_req_set_esi(IN osm_sm_t *p_sm,
+				IN osm_switch_t *p_sw,
+				IN ib_extended_switch_info_t *p_esi);
+/*
+* PARAMETERS
+*	p_sm
+*		[in] Pointer to an osm_sm_t object.
+*
+*	p_sw
+*		[in] Pointer to an osm_switch_t object.
+*
+*	p_esi
+*		[in] Pointer to the ExtendedSwitchInfo attribute.
+*
+* RETURN VALUE
+*	ib_api_status_t of the request
+*
+* NOTES
+*
+*********/
 
 /****f* OpenSM: SM/osm_prepare_req_get_pi
 * NAME
@@ -1078,6 +1175,9 @@ osm_madw_t *osm_prepare_req_set_pi(IN osm_sm_t * sm, IN const osm_physp_t *p_phy
 * DESCRIPTION
 *	Starts the process to transmit a directed route response.
 *
+* Note: This code is not thread safe. Need to grab the lock before
+* calling it.
+*
 * SYNOPSIS
 */
 ib_api_status_t osm_resp_send(IN osm_sm_t * sm,
