@@ -1,6 +1,6 @@
-MFT_VERSION_STR="mft 4.21.0-102"
-TOOLS_BUILD_TIME="Aug 18 2022, 21:58:43"
-TOOLS_GIT_SHA="f1a12d7"
+MFT_VERSION_STR="mft 4.30.1-1210"
+TOOLS_BUILD_TIME="Jun 12 2025, 18:01:27"
+TOOLS_GIT_SHA="N/A"
 
 function print_version_string {
 	tool_name=$1
