@@ -186,7 +186,7 @@ try:
             print("-E- Dump file option is not supported for Windows")
             sys.exit(1)
         wqdump_all_cmd = 'cat ' + options.file
-        (MCG_ENTRY_SIZE, HASH_TABLE_SIZE, MCG_TABLE_SIZE) = eval(options.params)
+        (MCG_ENTRY_SIZE, HASH_TABLE_SIZE, MCG_TABLE_SIZE) = [int(item.strip(" ()"), 0) for item in options.params.split(",")]
 
     print("MCG table size:  %d K entries, Hash size:  %d K entries, Entry size:  %d B" % ((MCG_TABLE_SIZE / 1024), (HASH_TABLE_SIZE / 1024), MCG_ENTRY_SIZE))
     l4_prot_map = {0: "--", 3: "Other", 5: "UDP", 6: "TCP"}
