@@ -25,15 +25,15 @@
 
 #define ORTE_MAJOR_VERSION 4
 #define ORTE_MINOR_VERSION 1
-#define ORTE_RELEASE_VERSION 2
-#define ORTE_GREEK_VERSION "a1"
+#define ORTE_RELEASE_VERSION 7
+#define ORTE_GREEK_VERSION "rc1"
 #define ORTE_WANT_REPO_REV @ORTE_WANT_REPO_REV@
-#define ORTE_REPO_REV "v4.1.1-30-g535358e"
+#define ORTE_REPO_REV "v4.1.5-165-ge4d9b162ee"
 #ifdef ORTE_VERSION
 /* If we included version.h, we want the real version, not the
    stripped (no-r number) verstion */
 #undef ORTE_VERSION
 #endif
-#define ORTE_VERSION "4.1.2a1"
+#define ORTE_VERSION "4.1.7rc1"
 
 #endif
